"""
Tests
"""

import pytest

from ArWikiCats import resolve_label_ar

fix_title_all = False


test_historic = {
    "railway stations on national-register-of-historic-places": "محطات السكك الحديدية في السجل الوطني للأماكن التاريخية",
    "residential buildings on national-register-of-historic-places": "مبان سكنية في السجل الوطني للأماكن التاريخية",
    "opera houses on national-register-of-historic-places": "دور أوبرا في السجل الوطني للأماكن التاريخية",
    "military facilities on national-register-of-historic-places": "مرافق عسكرية في السجل الوطني للأماكن التاريخية",
    "museums on national-register-of-historic-places": "متاحف في السجل الوطني للأماكن التاريخية",
    "fire stations on national-register-of-historic-places": "محطات الإطفاء في السجل الوطني للأماكن التاريخية",
    "forts on national-register-of-historic-places": "حصون في السجل الوطني للأماكن التاريخية",
    "houses on national-register-of-historic-places": "منازل في السجل الوطني للأماكن التاريخية",
    "cemeteries on national-register-of-historic-places": "مقابر في السجل الوطني للأماكن التاريخية",
    "buildings and structures on national-register-of-historic-places": "مبان ومنشآت في السجل الوطني للأماكن التاريخية",
    "bank buildings on national-register-of-historic-places": "مباني بنوك في السجل الوطني للأماكن التاريخية",
    "archaeological sites on national-register-of-historic-places": "مواقع أثرية في السجل الوطني للأماكن التاريخية",
    "agricultural buildings and structures on national-register-of-historic-places": "مبان ومنشآت زراعية في السجل الوطني للأماكن التاريخية",
    "clubhouses on national-register-of-historic-places": "نوادي في السجل الوطني للأماكن التاريخية",
    "courthouses on national-register-of-historic-places": "محاكم في السجل الوطني للأماكن التاريخية",
}

data_fast = {
    "kingdom-of italy": "مملكة إيطاليا",
    "11th century": "القرن 11",
    "12th century": "القرن 12",
    "1330 by country": "1330 حسب البلد",
    "1330 elections in united states": "انتخابات 1330 في الولايات المتحدة",
    "1330 in canada": "كندا في 1330",
    "1330 in china": "الصين في 1330",
    "1330 in football": "كرة القدم في 1330",
    "1330 in men's football": "كرة القدم للرجال في 1330",
    "1330 in north america": "أمريكا الشمالية في 1330",
    "1330 in rugby union": "اتحاد الرجبي في 1330",
    "1330 in south american football": "كرة القدم الأمريكية الجنوبية في 1330",
    "1330 in sports": "ألعاب رياضية في 1330",
    "1330 in taiwan": "تايوان في 1330",
    "1330 in television": "التلفاز في 1330",
    "1330 in united states": "الولايات المتحدة في 1330",
    "1330 in yemen": "اليمن في 1330",
    "1330 in youth football": "كرة القدم للشباب في 1330",
    "1330s in united states": "الولايات المتحدة في عقد 1330",
    "24th century": "القرن 24",
    "actors by religion": "ممثلون حسب الدين",
    "air force": "قوات جوية",
    "al bayda governorate": "محافظة البيضاء (اليمن)",
    "al rayyan": "الريان",
    "alabama politicians": "سياسيو ألاباما",
    "alvark tokyo": "تويوتا ألفارك طوكيو",
    "amateur football competitions": "منافسات كرة قدم للهواة",
    "ambassadors of afghanistan": "سفراء أفغانستان",
    "ambassadors of algeria": "سفراء الجزائر",
    "ambassadors to ottoman empire": "سفراء لدى الدولة العثمانية",
    "american television series basedon british television series": "مسلسلات تلفزيونية أمريكية مبنية على مسلسلات تلفزيونية بريطانية",
    "april 1330": "أبريل 1330",
    "archaeological sites on national-register-of-historic-places": "مواقع أثرية في السجل الوطني للأماكن التاريخية",
    "architecture in israel": "هندسة معمارية في إسرائيل",
    "armenia–colombia relations": "العلاقات الأرمينية الكولومبية",
    "asian football by country": "كرة القدم الآسيوية حسب البلد",
    "athletics competitions": "منافسات ألعاب قوى",
    "attacks on barracks": "هجمات على ثكنات",
    "attacks on buildings and structures": "هجمات على مبان ومنشآت",
    "attacks on diplomatic missions": "هجمات على بعثات دبلوماسية",
    "attacks on hospitals": "هجمات على مستشفيات",
    "attacks on malacañang palace": "هجمات على قصر مالاكانانغ",
    "august 1330": "أغسطس 1330",
    "australia by century": "أستراليا حسب القرن",
    "australian rules football clubs": "أندية كرة قدم أسترالية",
    "austria by decade": "النمسا حسب العقد",
    "badminton venues": "ملاعب تنس ريشة",
    "bahamas–ivory coast relations": "العلاقات الإيفوارية البهامية",
    "bahrain–netherlands relations": "العلاقات البحرينية الهولندية",
    "balakan district": "مقاطعة بالاكن",
    "bangladesh–iraq relations": "العلاقات البنغلاديشية العراقية",
    "bank of korea": "بنك كوريا",
    "barracks in spain": "ثكنات في إسبانيا",
}

data_slow = {
    "people from santa fe province": "أشخاص من محافظة سانتا في",
    # "presidents of greece": "رؤساء اليونان",
    # "presidents of maldives": "رؤساء جزر المالديف",
    "bas-sassandra district": "مقاطعة باس-ساساندرا",
    "baseball players from maine": "لاعبو كرة قاعدة من مين",
    "basketball competitions": "منافسات كرة سلة",
    "basketball people": "أعلام كرة سلة",
    "belgium by decade": "بلجيكا حسب العقد",
    "belgium–czechoslovakia relations": "العلاقات البلجيكية التشيكوسلوفاكية",
    "books about automobiles": "كتب عن سيارات",
    "books about politics": "كتب عن سياسة",
    "boxing competitions": "منافسات بوكسينغ",
    "brain cancer in south korea": "سرطان الدماغ في كوريا الجنوبية",
    "brazilian comics adapted into films": "قصص مصورة برازيلية تم تحويلها إلى أفلام",
    "bridges in wales": "جسور في ويلز",
    "british emigrants to switzerland": "بريطانيون مهاجرون إلى سويسرا",
    "british emigrants to zimbabwe": "بريطانيون مهاجرون إلى زيمبابوي",
    "british television series characters by series": "شخصيات مسلسلات تلفزيونية بريطانية حسب السلسلة",
    "buildings and structures in 1330": "مبان ومنشآت في 1330",
    "buildings and structures in australia": "مبان ومنشآت في أستراليا",
    "buildings and structures in chile": "مبان ومنشآت في تشيلي",
    "buildings and structures in florida": "مبان ومنشآت في فلوريدا",
    "buildings and structures in quebec": "مبان ومنشآت في كيبك",
    "buildings and structures in united arab emirates": "مبان ومنشآت في الإمارات العربية المتحدة",
    "buildings and structures in united states": "مبان ومنشآت في الولايات المتحدة",
    "buildings and structures in veneto": "مبان ومنشآت في فينيتو",
    "bulgaria–qatar relations": "العلاقات البلغارية القطرية",
    "burials in amsterdam": "مدفونون في أمستردام",
    "by city in afghanistan": "حسب المدينة في أفغانستان",
    "by city in jharkhand": "حسب المدينة في جهارخاند",
    "by instrument from northern ireland": "حسب الآلة من أيرلندا الشمالية",
    "by period by state": "حسب الحقبة حسب الولاية",
    "by person in alabama": "حسب الشخص في ألاباما",
    "by person in austria": "حسب الشخص في النمسا",
    "by person in tonga": "حسب الشخص في تونغا",
    "by province of saudi arabia": "حسب المقاطعة في السعودية",
    "by stabbing in philippines": "بالطعن في الفلبين",
    "by state in united states": "حسب الولاية في الولايات المتحدة",
    "by university or college in ghana": "حسب الجامعة أو الكلية في غانا",
    "cabinet of philippines": "مجلس وزراء الفلبين",
    "campeonato paraibano": "بطولة بارايبانو",
    "canada by city": "كندا حسب المدينة",
    "canada by month": "كندا حسب الشهر",
    "canada–gabon relations": "العلاقات الغابونية الكندية",
    "cancer in hungary": "السرطان في المجر",
    "cancer in philippines": "السرطان في الفلبين",
    "cancer in singapore": "السرطان في سنغافورة",
    "canoeing and kayaking competitions": "منافسات كانو-كاياك",
    "cape verde": "الرأس الأخضر",
    "categories of haryana": "تصنيفات في هاريانا",
    "caves in india": "كهوف في الهند",
    "cemeteries on national-register-of-historic-places": "مقابر في السجل الوطني للأماكن التاريخية",
    "centuries in united states": "قرون في الولايات المتحدة",
    "china by month": "الصين حسب الشهر",
    "chinese emigrants to thailand": "صينيون مهاجرون إلى تايلاند",
    "christina aguilera video albums": "ألبومات فيديو كريستينا أغيليرا",
    "churches in france": "كنائس في فرنسا",
    "churches in nicaragua": "كنائس في نيكاراغوا",
    "colombia–united states relations": "العلاقات الأمريكية الكولومبية",
    "comics adapted into films": "قصص مصورة تم تحويلها إلى أفلام",
    "comics basedon films": "قصص مصورة مبنية على أفلام",
    "commonwealth games weightlifters for england": "رباعون في ألعاب الكومنولث في إنجلترا",
    "communications in united states": "الاتصالات في الولايات المتحدة",
    "companies of bahamas": "شركات في باهاماس",
    "constructivism (art)": "بنائية (فنون)",
    "county longford": "مقاطعة لونجفورد",
    "cricket clubs": "أندية كريكت",
    "crimes in belgium": "جرائم في بلجيكا",
    "crimes in united states": "جرائم في الولايات المتحدة",
    "croatian emigrants to netherlands": "كروات مهاجرون إلى هولندا",
    "cycling clubs": "أندية سباق دراجات هوائية",
    "cypriot emigrants to israel": "قبرصيون مهاجرون إلى إسرائيل",
    "deaths from covid-19 pandemic": "وفيات من جائحة فيروس كورونا",
    "deaths from tuberculosis": "وفيات من السل",
    "decades in united states": "عقود في الولايات المتحدة",
    "december 1330": "ديسمبر 1330",
    "defunct football cup competitions": "منافسات كؤوس كرة قدم سابقة",
    "defunct ice hockey competitions": "منافسات هوكي جليد سابقة",
    "demographics of united states": "التركيبة السكانية في الولايات المتحدة",
    "denmark–vietnam relations": "العلاقات الدنماركية الفيتنامية",
    "disasters in united states": "كوارث في الولايات المتحدة",
    "disestablishments in australia": "انحلالات في أستراليا",
    "disestablishments in dominica": "انحلالات في دومينيكا",
    "disestablishments in new zealand": "انحلالات في نيوزيلندا",
    "disestablishments in ottoman empire": "انحلالات في الدولة العثمانية",
    "disestablishments in papua new guinea": "انحلالات في بابوا غينيا الجديدة",
    "disestablishments in spain": "انحلالات في إسبانيا",
    "documentaries about crime": "وثائقيات عن جريمة",
    "documentary films about federico fellini": "أفلام وثائقية عن فيديريكو فليني",
    "documentary films about south carolina": "أفلام وثائقية عن كارولاينا الجنوبية",
    "domestic football": "كرة قدم محلية",
    "dominica by year": "دومينيكا حسب السنة",
    "dukes in europe": "دوقات في أوروبا",
    "dutch basketball league by club": "الدوري الهولندي لكرة السلة حسب النادي",
    "economic history of united states": "تاريخ الولايات المتحدة الاقتصادي",
    "economy of united states": "اقتصاد الولايات المتحدة",
    "ecuador–guyana relations": "العلاقات الإكوادورية الغيانية",
    "ed sheeran albums": "ألبومات إد شيران",
    "education in france": "التعليم في فرنسا",
    "education in israel": "التعليم في إسرائيل",
    "education in united states": "التعليم في الولايات المتحدة",
    "elections in austria": "انتخابات في النمسا",
    "electro house remix albums": "ألبومات ريمكس إلكتروهاوس",
    "environment of united states": "بيئة الولايات المتحدة",
    "eritrea–ukraine relations": "العلاقات الأوكرانية الإريترية",
    "establishments in north macedonia": "تأسيسات في مقدونيا الشمالية",
    "establishments in republic-of venice": "تأسيسات في جمهورية البندقية",
    "establishments in sint maarten": "تأسيسات في سينت مارتن",
    "establishments in socialist republic-of macedonia": "تأسيسات في جمهورية مقدونيا الاشتراكية",
    "estonia–south sudan relations": "العلاقات الإستونية السودانية الجنوبية",
    "ethiopian basketball players": "لاعبو كرة سلة إثيوبيون",
    "euroleague by club": "الدوري الأوروبي لكرة السلة حسب النادي",
    "european games gold medalists for slovakia": "فائزون بميداليات ذهبية في الألعاب الأوروبية من سلوفاكيا",
    "european games medalists for poland": "فائزون بميداليات الألعاب الأوروبية من بولندا",
    "european rugby union by country": "اتحاد الرجبي الأوروبي حسب البلد",
    "executions in washington (state)": "إعدامات في ولاية واشنطن",
    "expatriate sports-people from northern ireland": "رياضيون مغتربون من أيرلندا الشمالية",
    "expatriates from georgia (country)": "مغتربون من جورجيا",
    "experimental music remix albums": "ألبومات ريمكس موسيقى تجريبية",
    "explosions in europe": "انفجارات في أوروبا",
    "fauna of africa": "حيوانات إفريقيا",
    "february 1330": "فبراير 1330",
    "figure skating films": "أفلام تزلج فني",
    "figure skating media": "إعلام تزلج فني",
    "figure skating people": "أعلام تزلج فني",
    "figure skating": "التزلج الفني",
    "films about automobiles": "أفلام عن سيارات",
    "films about chinese canadians": "أفلام عن صينيون كنديون",
    "films about higher education": "أفلام عن تعليم عال",
    "films about lizards": "أفلام عن سحالي",
    "films about olympic boxing": "أفلام عن بوكسينغ أولمبي",
    "films about olympic figure skating": "أفلام عن تزلج فني أولمبي",
    "films about olympic games": "أفلام عن الألعاب الأولمبية",
    "films about olympic gymnastics": "أفلام عن جمباز أولمبي",
    "films about olympic skiing": "أفلام عن تزلج أولمبي",
    "films about olympic track and field": "أفلام عن سباقات مضمار وميدان أولمبي",
    "films about weather": "أفلام عن الطقس",
    "films basedon american comics": "أفلام مبنية على قصص مصورة أمريكية",
    "films basedon comics": "أفلام مبنية على قصص مصورة",
    "films basedon television series": "أفلام مبنية على مسلسلات تلفزيونية",
    "films basedon works": "أفلام مبنية على أعمال",
    "films in yemen": "أفلام في اليمن",
    "finland–united states relations": "العلاقات الأمريكية الفنلندية",
    "fire stations on national-register-of-historic-places": "محطات الإطفاء في السجل الوطني للأماكن التاريخية",
    "football by country": "كرة القدم حسب البلد",
    "football cup competitions in england": "منافسات كؤوس كرة قدم في إنجلترا",
    "football cup competitions": "منافسات كؤوس كرة قدم",
    "football people": "أعلام كرة قدم",
    "football players by national team": "لاعبو كرة قدم حسب المنتخب الوطني",
    "forts in united states": "حصون في الولايات المتحدة",
    "forts on national-register-of-historic-places": "حصون في السجل الوطني للأماكن التاريخية",
    "gaelic football competitions": "منافسات كرة قدم غالية",
    "gaelic football": "كرة القدم الغالية",
    "gaelic footballers in county kildare": "لاعبو كرة قدم غيلية في مقاطعة كيلدير",
    "geography of tokyo": "جغرافيا طوكيو",
    "geography of united states": "جغرافيا الولايات المتحدة",
    "georgia (u.s. state)": "ولاية جورجيا",
    "german emigrants to scotland": "ألمان مهاجرون إلى إسكتلندا",
    "german footballers": "لاعبو كرة قدم ألمان",
    "german novels of 20th century": "روايات ألمانية في القرن 20",
    "golf in united states": "الغولف في الولايات المتحدة",
    "government ministers of azerbaijan": "وزراء أذربيجان",
    "great britain": "بريطانيا العظمى",
    "gymnastics competitions": "منافسات جمباز",
    "gymnastics organizations": "منظمات جمباز",
    "handball competitions": "منافسات كرة يد",
    "heads of schools": "قادة مدارس",
    "heads of universities and colleges": "قادة جامعات وكليات",
    "health in united states": "الصحة في الولايات المتحدة",
    "heavy metal video albums": "ألبومات فيديو هيفي ميتال",
    "high schools in illinois": "مدارس ثانوية في إلينوي",
    "hindu temples in andhra pradesh": "معابد هندوسية في أندرا برديش",
    "historic sites in united states": "مواقع تاريخية في الولايات المتحدة",
    "historic trails and roads in united states": "طرق وممرات تاريخية في الولايات المتحدة",
    "history books about printing": "كتب تاريخ عن طباعة",
    "history of american revolution": "تاريخ الثورة الأمريكية",
    "history of caribbean": "تاريخ الكاريبي",
    "history of human rights": "تاريخ حقوق الإنسان",
    "history of united states": "تاريخ الولايات المتحدة",
    "history of women": "تاريخ المرأة",
    "hospitals in uganda": "مستشفيات في أوغندا",
    "houses on national-register-of-historic-places": "منازل في السجل الوطني للأماكن التاريخية",
    "hungarian people imprisoned-in germany": "مجريون مسجونون في ألمانيا",
    "ice hockey people": "أعلام هوكي جليد",
    "ice skating": "تزلج على الجليد",
    "icelandic emigrants to new zealand": "آيسلنديون مهاجرون إلى نيوزيلندا",
    "images of united states": "صور من الولايات المتحدة",
    "immigrants to american samoa": "مهاجرون إلى ساموا الأمريكية",
    "immigrants to new zealand": "مهاجرون إلى نيوزيلندا",
    "immigration to new zealand": "الهجرة إلى نيوزيلندا",
    "indian cricketers": "لاعبو كريكت هنود",
    "insular areas of united states": "المناطق المعزولة في الولايات المتحدة",
    "international aquatics competitions": "منافسات رياضات مائية دولية",
    "international athletics competitions": "منافسات ألعاب قوى دولية",
    "international baseball competitions": "منافسات كرة قاعدة دولية",
    "international field hockey competitions in africa": "منافسات هوكي ميدان دولية في إفريقيا",
    "international field hockey competitions": "منافسات هوكي ميدان دولية",
    "international football": "كرة قدم دولية",
    "international schools in united arab emirates": "مدارس دولية في الإمارات العربية المتحدة",
    "international women's basketball competitions": "منافسات كرة سلة دولية للسيدات",
    "iowa territory": "إقليم آيوا",
    "islands of greece": "جزر اليونان",
    "italian defectors to soviet union": "إيطاليون منشقون إلى الاتحاد السوفيتي",
    "italian emigrants to finland": "إيطاليون مهاجرون إلى فنلندا",
    "italy–liechtenstein relations": "العلاقات الإيطالية الليختنشتانية",
    "january 1330": "يناير 1330",
    "japan–switzerland relations": "العلاقات السويسرية اليابانية",
    "jews from ottoman empire": "يهود من الدولة العثمانية",
    "july 1330": "يوليو 1330",
    "june 1330": "يونيو 1330",
    "kansas house-of-representatives": "مجلس نواب ولاية كانساس",
    "kentucky ballot measures": "إجراءات اقتراع كنتاكي",
    "kingdom-of italy": "مملكة إيطاليا",
    "kingdom-of sicily": "مملكة صقلية",
    "kings of italy": "ملوك إيطاليا",
    "kosovan victims of crime": "ضحايا جرائم كوسوفيون",
    "kuwait–turkey relations": "العلاقات التركية الكويتية",
    "kütahya province": "محافظة كوتاهية",
    "landmarks in united states": "معالم في الولايات المتحدة",
    "las vegas": "لاس فيغاس (نيفادا)",
    "latvia–uruguay relations": "العلاقات الأوروغويانية اللاتفية",
    "law about religion": "قانون عن الدين",
    "lawn bowls": "البولينغ العشبي",
    "learned societies of china": "جمعيات علمية في الصين",
    "legal events": "أحداث قانونية",
    "legal history of united states": "تاريخ الولايات المتحدة القانوني",
    "liechtenstein football": "كرة القدم الليختنشتانية",
    "lists of 1330 films": "قوائم أفلام في 1330",
    "lists of british television series characters": "قوائم شخصيات مسلسلات تلفزيونية بريطانية",
    "lists of buildings and structures": "قوائم مبان ومنشآت",
    "lists of caves": "قوائم كهوف",
    "lists of countries": "قوائم بلدان",
    "lists of football players": "قوائم لاعبو كرة قدم",
    "lists of german people": "قوائم ألمان",
    "lists of mayors of places": "قوائم رؤساء بلديات",
    "lists of schools": "قوائم مدارس",
    "lists of television characters": "قوائم شخصيات تلفزيونية",
    "lists of tourist attractions": "قوائم مواقع جذب سياحي",
    "lists of universities and colleges": "قوائم جامعات وكليات",
    "loire valley": "وادي اللوار",
    "mackinac island": "جزيرة ماكيناك",
    "mahasamund district": "مقاطعة مهاساموند",
    "malaysia in fiction": "ماليزيا في الخيال",
    "malaysia in summer olympics": "ماليزيا في الألعاب الأولمبية الصيفية",
    "manufacturing in united states": "تصنيع في الولايات المتحدة",
    "marathi films remade-in other languages": "أفلام باللغة الماراثية أعيد إنتاجها في لغات أخرى",
    "march 1330": "مارس 1330",
    "martial arts": "الفنون القتالية",
    "mass shootings in europe": "إطلاق نار عشوائي في أوروبا",
    "massachusetts know nothings": "أعضاء حزب لا أدري في ماساتشوستس",
    "matn district": "قضاء المتن",
    "may 1330": "مايو 1330",
    "mayors of places in alabama": "رؤساء بلديات في ألاباما",
    "mayors of places in estonia": "رؤساء بلديات في إستونيا",
    "mayors of places in new jersey": "رؤساء بلديات في نيوجيرسي",
    "mecha anime and manga": "أنمي ميكا",
    "medley swimming": "سباحة متنوعة",
    "members of pan-african parliament": "أعضاء البرلمان الإفريقي",
    "men in united states": "رجال في الولايات المتحدة",
    "men's basketball": "كرة السلة للرجال",
    "men's football by continent": "كرة القدم للرجال حسب القارة",
    "men's football competitions": "منافسات كرة قدم رجالية",
    "men's football": "كرة القدم للرجال",
    "men's footballers in djibouti": "لاعبو كرة قدم في جيبوتي",
    "men's footballers in montenegro": "لاعبو كرة قدم في الجبل الأسود",
    "men's soccer": "كرة القدم للرجال",
    "michigan legislature": "هيئة ميشيغان التشريعية",
    "michigan territory": "إقليم ميشيغان",
    "mike & molly": "مايك ومولي",
    "military alliances involving algeria": "تحالفات عسكرية تشمل الجزائر",
    "military history of africa": "تاريخ إفريقيا العسكري",
    "military history of croatia": "تاريخ كرواتيا العسكري",
    "military history of united states": "تاريخ الولايات المتحدة العسكري",
    "military pentathlon": "خماسي عسكرية",
    "mississippi territory": "إقليم مسيسيبي",
    "mixed martial arts": "فنون القتال المختلطة",
    "moldova–south korea relations": "العلاقات الكورية الجنوبية المولدوفية",
    "monmouth university": "جامعة مونموث",
    "moose jaw": "موسجاو",
    "motorsport competitions": "منافسات رياضة محركات",
    "mountains of greece": "جبال اليونان",
    "muay thai": "موياي تاي",
    "murders by country": "جرائم قتل حسب البلد",
    "murders in asia": "جرائم قتل في آسيا",
    "murders in belarus": "جرائم قتل في روسيا البيضاء",
    "murders in iraq": "جرائم قتل في العراق",
    "murders in philippines": "جرائم قتل في الفلبين",
    "murders in republic-of ireland": "جرائم قتل في جمهورية أيرلندا",
    "murders in south africa": "جرائم قتل في جنوب إفريقيا",
    "murders in spain": "جرائم قتل في إسبانيا",
    "murders in united kingdom": "جرائم قتل في المملكة المتحدة",
    "museums in north carolina": "متاحف في كارولاينا الشمالية",
    "museums of ancient greece": "متاحف في اليونان القديمة",
    "museums of ancient rome": "متاحف في روما القديمة",
    "museums on national-register-of-historic-places": "متاحف في السجل الوطني للأماكن التاريخية",
    "music of jean sibelius": "موسيقى جان سيبيليوس",
    "nascar races": "سباقات ناسكار",
    "national cricket teams": "منتخبات كريكت وطنية",
    "national wheelchair rugby league teams": "منتخبات دوري رجبي على كراسي متحركة وطنية",
    "nature reserves in united states": "محميات طبيعية في الولايات المتحدة",
    "naval battles involving united states": "معارك بحرية تشمل الولايات المتحدة",
    "navoiy region": "ولاية نواوي",
    "nazis sentenced to death": "نازيون حكم عليهم بالإعدام",
    "nba season by team": "الدوري الأمريكي لكرة السلة للمحترفين حسب الفريق",
    "new hampshire": "نيوهامشير",
    "new jersey": "نيوجيرسي",
    "new mexico": "نيومكسيكو",
    "new york (state)": "ولاية نيويورك",
    "new zealand by decade": "نيوزيلندا حسب العقد",
    "nobility from holy roman empire": "نبلاء من الإمبراطورية الرومانية المقدسة",
    "nordic combined": "التزلج النوردي المزدوج",
    "north america by month": "أمريكا الشمالية حسب الشهر",
    "north carolina": "كارولاينا الشمالية",
    "north dakota": "داكوتا الشمالية",
    "north macedonia by century": "مقدونيا الشمالية حسب القرن",
    "northern ireland of canadian descent": "أيرلندا الشمالية من أصل كندي",
    "november 1330": "نوفمبر 1330",
    "october 1330": "أكتوبر 1330",
    "olympic archers for jordan": "نبالون أولمبيون في الأردن",
    "olympic biathletes for lithuania": "لاعبو بياثلون أولمبيون في ليتوانيا",
    "olympic boxers for australia": "ملاكمون أولمبيون في أستراليا",
    "olympic competitors for cape verde": "منافسون أولمبيون من الرأس الأخضر",
    "olympic competitors for guam": "منافسون أولمبيون من غوام",
    "olympic equestrians for cambodia": "فرسان خيول أولمبيون في كمبوديا",
    "olympic fencers for china": "مبارزون أولمبيون في الصين",
    "olympic figure skating": "تزلج فني أولمبي",
    "olympic gold medalists for south africa": "فائزون بميداليات ذهبية أولمبية من جنوب إفريقيا",
    "olympic gold medalists for united states": "فائزون بميداليات ذهبية أولمبية من الولايات المتحدة",
    "olympic ice hockey players for belgium": "لاعبو هوكي جليد أولمبيون في بلجيكا",
    "olympic judoka for italy": "لاعبو جودو أولمبيون في إيطاليا",
    "olympic medalists for united states": "فائزون بميداليات أولمبية من الولايات المتحدة",
    "olympic medalists": "فائزون بميداليات أولمبية",
    "olympic skeleton racers for great britain": "متزلجون صدريون أولمبيون في بريطانيا العظمى",
    "olympic swimming": "سباحة أولمبية",
    "olympic wheelchair racers for australia": "متسابقو كراسي متحركة أولمبيون في أستراليا",
    "olympic wheelchair racers for canada": "متسابقو كراسي متحركة أولمبيون في كندا",
    "organizations based in overseas france": "منظمات مقرها في مقاطعات وأقاليم ما وراء البحار الفرنسية",
    "organizations based in turks and caicos islands": "منظمات مقرها في جزر توركس وكايكوس",
    "ottoman–persian wars": "الحروب العثمانية الفارسية",
    "painters from holy roman empire": "رسامون من الإمبراطورية الرومانية المقدسة",
    "pan american games medalists for bermuda": "فائزون بميداليات دورة الألعاب الأمريكية من برمودا",
    "pan american games medalists for guyana": "فائزون بميداليات دورة الألعاب الأمريكية من غيانا",
    "pan american games medalists": "فائزون بميداليات دورة الألعاب الأمريكية",
    "pan american games swimmers for brazil": "سباحون في دورة الألعاب الأمريكية في البرازيل",
    "pan-african democratic party politicians": "سياسيو الحزب الديمقراطي الوحدوي الإفريقي",
    "pan-african parliament": "البرلمان الإفريقي",
    "pan-africanist political parties": "أحزاب سياسية وحدوية إفريقية",
    "papua new guinea by year": "بابوا غينيا الجديدة حسب السنة",
    "paralympic athletes for colombia": "لاعبو قوى في الألعاب البارالمبية في كولومبيا",
    "paralympic judoka for uzbekistan": "لاعبو جودو في الألعاب البارالمبية في أوزبكستان",
    "paralympic medalists for serbia": "فائزون بميداليات الألعاب البارالمبية من صربيا",
    "paralympic medalists": "فائزون بميداليات الألعاب البارالمبية",
    "paralympic wheelchair basketball players for australia": "لاعبو كرة سلة على كراسي متحركة في الألعاب البارالمبية في أستراليا",
    "paralympic wheelchair basketball players for canada": "لاعبو كرة سلة على كراسي متحركة في الألعاب البارالمبية في كندا",
    "paralympic wheelchair rugby players for new zealand": "لاعبو رجبي على كراسي متحركة في الألعاب البارالمبية في نيوزيلندا",
    "paralympic wheelchair rugby players for united states": "لاعبو رجبي على كراسي متحركة في الألعاب البارالمبية في الولايات المتحدة",
    "parapan american games medalists": "فائزون بميداليات ألعاب بارابان الأمريكية",
    "parliament for loiret": "برلمان في لواريت (إقليم فرنسي)",
    "parliament of england 1330": "برلمان إنجلترا 1330",
    "parliament of united kingdom": "برلمان المملكة المتحدة",
    "people by religion": "أشخاص حسب الدين",
    "people from asturias": "أشخاص من منطقة أستورياس",
    "people from hawaii": "أشخاص من هاواي",
    "people from madhya pradesh": "أشخاص من ماديا براديش",
    "people from ottoman empire": "أشخاص من الدولة العثمانية",
    "people from saxony": "أشخاص من ساكسونيا",
    "people from south lanarkshire": "أشخاص من جنوب لاناركشاير",
    "people from westchester county, new york": "أشخاص من مقاطعة ويستتشستر (نيويورك)",
    "people of dutch empire": "أشخاص من الإمبراطورية الهولندية",
    "people of el salvador": "أشخاص من السلفادور",
    "people of nicaragua": "أشخاص من نيكاراغوا",
    "perennial philosophy": "فلسفة خالدة",
    "permanent representatives of jamaica": "مندوبو جامايكا الدائمون",
    "philmont scout ranch": "مزرعة فيلمونت الكشفية",
    "physicians from bohemia": "أطباء من بوهيميا",
    "polish emigrants to brazil": "بولنديون مهاجرون إلى البرازيل",
    "political history of united states": "تاريخ الولايات المتحدة السياسي",
    "politics of united states": "سياسة الولايات المتحدة",
    "populated places in latvia": "أماكن مأهولة في لاتفيا",
    "populated places in portugal": "أماكن مأهولة في البرتغال",
    "populated places in uruguay": "أماكن مأهولة في أوروغواي",
    "prime ministers of malaysia": "رؤساء وزراء ماليزيا",
    "protected areas of united states": "مناطق محمية في الولايات المتحدة",
    "provinces of saudi arabia": "مقاطعات السعودية",
    "puerto rico": "بورتوريكو",
    "rail transport in sri lanka": "السكك الحديدية في سريلانكا",
    "railway stations in australia": "محطات السكك الحديدية في أستراليا",
    "railway stations in cambodia": "محطات السكك الحديدية في كمبوديا",
    "railway stations on national-register-of-historic-places": "محطات السكك الحديدية في السجل الوطني للأماكن التاريخية",
    "ray tracing (graphics)": "اقتفاء الشعاع",
    "religious buildings and structures in trinidad and tobago": "مبان ومنشآت دينية في ترينيداد وتوباغو",
    "rhode island": "رود آيلاند",
    "rhodope mountains": "جبال رودوب",
    "riots and civil disorder in united states": "شغب وعصيان مدني في الولايات المتحدة",
    "rivers of brazil": "أنهار البرازيل",
    "road incident deaths in washington (state)": "وفيات حوادث الطرق في ولاية واشنطن",
    "roman catholic archbishops in austria": "رؤساء أساقفة رومان كاثوليك في النمسا",
    "roman catholic archbishops in ireland": "رؤساء أساقفة رومان كاثوليك في أيرلندا",
    "roman catholic archbishops in singapore": "رؤساء أساقفة رومان كاثوليك في سنغافورة",
    "roman catholic church buildings in italy": "مبان كنائس رومانية كاثوليكية في إيطاليا",
    "roman catholic church buildings in malta": "مبان كنائس رومانية كاثوليكية في مالطا",
    "rowing clubs": "أندية تجديف",
    "rugby league in england": "دوري الرجبي في إنجلترا",
    "rugby union by country": "اتحاد الرجبي حسب البلد",
    "rugby union competitions": "منافسات اتحاد رجبي",
    "rugby union tournaments": "بطولات اتحاد رجبي",
    "russia–sri lanka relations": "العلاقات الروسية السريلانكية",
    "rwanda–tanzania relations": "العلاقات التنزانية الرواندية",
    "sailing for puerto rico": "الإبحار في بورتوريكو",
    "school districts in california": "مناطق تعليمية في كاليفورنيا",
    "schools in delaware": "مدارس في ديلاوير",
    "science and technology in united states": "العلوم والتقانة في الولايات المتحدة",
    "science fiction literature": "أدب خيال علمي",
    "scientists from ottoman empire": "علماء من الدولة العثمانية",
    "sculptures of men": "منحوتات في رجال",
    "september 1330": "سبتمبر 1330",
    "sexual violence in asia": "عنف جنسي في آسيا",
    "ships built-in montreal": "سفن بنيت في مونتريال",
    "shooting competitions": "منافسات رماية",
    "sieges involving austria": "حصارات تشمل النمسا",
    "sierra morena": "سييرا مورينا",
    "sint maarten by decade": "سينت مارتن حسب العقد",
    "ski jumping": "القفز التزلجي",
    "slavery in united states": "العبودية في الولايات المتحدة",
    "soccer matches": "مباريات كرة قدم",
    "socialist party-of serbia": "الحزب الاشتراكي في صربيا",
    "society of united states": "مجتمع الولايات المتحدة",
    "songs about automobiles": "أغاني عن سيارات",
    "songs about busan": "أغاني عن بوسان",
    "songs about new mexico": "أغاني عن نيومكسيكو",
    "songs about south dakota": "أغاني عن داكوتا الجنوبية",
    "south africa": "جنوب إفريقيا",
    "south american football by country": "كرة القدم الأمريكية الجنوبية حسب البلد",
    "south carolina politicians": "سياسيو كارولاينا الجنوبية",
    "south carolina": "كارولاينا الجنوبية",
    "south dakota politics": "سياسة داكوتا الجنوبية",
    "south dakota": "داكوتا الجنوبية",
    "south west africa": "جنوب غرب إفريقيا",
    "spain by decade": "إسبانيا حسب العقد",
    "speed skating": "التزلج السريع",
    "sports clubs and teams in brazil": "أندية وفرق رياضية في البرازيل",
    "sports clubs and teams": "أندية وفرق رياضية",
    "sports teams": "فرق رياضية",
    "sports-people from saxony": "رياضيون من ساكسونيا",
    "squares in london": "ساحات في لندن",
    "sri lankan emigrants to india": "سريلانكيون مهاجرون إلى الهند",
    "sri lanka–vietnam relations": "العلاقات السريلانكية الفيتنامية",
    "summer olympics competitors for canada": "منافسون أولمبيون صيفيون من كندا",
    "synagogues in france": "كنس في فرنسا",
    "sánchez ramírez province": "محافظة سانشيز راميريز",
    "table tennis clubs": "أندية كرة طاولة",
    "table tennis": "كرة الطاولة",
    "taiwan by month": "تايوان حسب الشهر",
    "taiwanese emigrants to argentina": "تايوانيون مهاجرون إلى الأرجنتين",
    "tanger-tetouan-al hoceima": "جهة طنجة تطوان الحسيمة",
    "telecommunications in asia": "الاتصالات عن بعد في آسيا",
    "television by country": "التلفاز حسب البلد",
    "television characters by series": "شخصيات تلفزيونية حسب السلسلة",
    "television episodes about american civil war": "حلقات تلفزيونية عن الحرب الأهلية الأمريكية",
    "television episodes about vietnam war": "حلقات تلفزيونية عن حرب فيتنام",
    "television in ivory coast": "التلفاز في ساحل العاج",
    "television series about chefs": "مسلسلات تلفزيونية عن طباخون",
    "television series about united states navy": "مسلسلات تلفزيونية عن البحرية الأمريكية",
    "television shows basedon french novels": "عروض تلفزيونية مبنية على روايات فرنسية",
    "tennis people": "أعلام كرة مضرب",
    "terrorist incidents in oceania": "حوادث إرهابية في أوقيانوسيا",
    "the canary islands": "جزر الكناري",
    "the city of": "مدينة",
    "the covid-19 pandemic in paraná (state)": "جائحة فيروس كورونا في ولاية بارانا",
    "the crown dependencies": "ملحقات التاج البريطاني",
    "the democratic-republic-of-congo": "جمهورية الكونغو الديمقراطية",
    "the mamluk sultanate": "مماليك مصر",
    "the ottoman empire by decade": "الدولة العثمانية حسب العقد",
    "the parliament of united kingdom": "برلمان المملكة المتحدة",
    "the roman republic": "الجمهورية الرومانية",
    "the russian empire": "الإمبراطورية الروسية",
    "the scarlet letter": "الحرف القرمزي",
    "the united kingdom": "المملكة المتحدة",
    "the united states by state or territory": "الولايات المتحدة حسب الولاية أو الإقليم",
    "the united states by state": "الولايات المتحدة حسب الولاية",
    "the united states in 1330": "الولايات المتحدة في 1330",
    "the valencian community": "منطقة بلنسية",
    "tourist attractions in nepal": "مواقع جذب سياحي في نيبال",
    "tourist attractions in new zealand": "مواقع جذب سياحي في نيوزيلندا",
    "tourist attractions in nicaragua": "مواقع جذب سياحي في نيكاراغوا",
    "tourist attractions in united states": "مواقع جذب سياحي في الولايات المتحدة",
    "track and field competitions": "منافسات سباقات مضمار وميدان",
    "transport museums in canada": "متاحف نقل في كندا",
    "transportation in united states": "النقل في الولايات المتحدة",
    "treaties extended to curaçao": "اتفاقيات امتدت إلى كوراساو",
    "ukraine–united states military relations": "العلاقات الأمريكية الأوكرانية العسكرية",
    "united kingdom from aden": "المملكة المتحدة من عدن",
    "united nations security council resolutions concerning eritrea": "قرارات مجلس الأمن التابع للأمم المتحدة بشأن إرتريا",
    "united states by state": "الولايات المتحدة حسب الولاية",
    "united states house-of-representatives from missouri territory": "مجلس النواب الأمريكي من إقليم ميزوري",
    "united states in 1330": "الولايات المتحدة في 1330",
    "universities and colleges in canada": "جامعات وكليات في كندا",
    "universities and colleges in italy": "جامعات وكليات في إيطاليا",
    "university of alabama": "جامعة ألاباما",
    "university of arkansas": "جامعة أركنساس",
    "university of kent": "جامعة كنت",
    "vallès occidental": "فالس أوكيدنتل",
    "victims of aviation accidents or incidents": "ضحايا حوادث طيران",
    "video games about diseases": "ألعاب فيديو عن الأمراض",
    "video games about slavery": "ألعاب فيديو عن العبودية",
    "video games basedon egyptian mythology": "ألعاب فيديو مبنية على أساطير مصرية",
    "video games basedon mythology": "ألعاب فيديو مبنية على أساطير",
    "volleyball clubs": "أندية كرة طائرة",
    "war in romania": "الحرب في رومانيا",
    "washington (state)": "ولاية واشنطن",
    "washington territory": "إقليم واشنطن",
    "water polo": "كرة الماء",
    "web colors": "ألوان الويب",
    "west bank premier league": "الدوري الفلسطيني الممتاز للضفة الغربية",
    "west virginia": "فرجينيا الغربية",
    "wheelchair basketball competitions between national teams": "منافسات كرة سلة على كراسي متحركة بين منتخبات وطنية",
    "wheelchair basketball competitions": "منافسات كرة سلة على كراسي متحركة",
    "wheelchair basketball leagues": "دوريات كرة سلة على كراسي متحركة",
    "wisconsin territory": "إقليم ويسكونسن",
    "women's bmx": "سباق البي إم إكس للسيدات",
    "women's football by country": "كرة القدم للسيدات حسب البلد",
    "women's football": "كرة القدم للسيدات",
    "women's volleyball": "كرة الطائرة للسيدات",
    "works about automobiles": "أعمال عن سيارات",
    "works about countries": "أعمال عن بلدان",
    "works about guinea-bissau": "أعمال عن غينيا بيساو",
    "works about obesity": "أعمال عن السمنة",
    "works about people": "أعمال عن أشخاص",
    "works adapted for other media": "أعمال تم تحويلها إلى وسائط أخرى",
    "works basedon mythology": "أعمال مبنية على أساطير",
    "world judo championships": "بطولة العالم للجودو",
    "yamanashi prefecture": "محافظة ياماناشي",
    "years in united states": "سنوات في الولايات المتحدة",
    "years of 20th century": "سنوات القرن 20",
    "years of 21st century": "سنوات القرن 21",
    "youth athletics competitions": "منافسات ألعاب قوى شبابية",
    "youth athletics": "ألعاب القوى للشباب",
    "youth football by continent": "كرة القدم للشباب حسب القارة",
    "youth football competitions": "منافسات كرة قدم شبابية",
    "youth football": "كرة القدم للشباب",
    "youth olympic gold medalists for turkey": "فائزون بميداليات ذهبية في الألعاب الأولمبية الشبابية من تركيا",
    "youth olympic gold medalists": "فائزون بميداليات ذهبية في الألعاب الأولمبية الشبابية",
    "youth sports competitions": "منافسات رياضية شبابية",
    "zambian emigrants to sweden": "زامبيون مهاجرون إلى السويد",
    "zürich eprix": "زيورخ إي بريكس",
}


@pytest.mark.parametrize("category, expected", data_fast.items(), ids=data_fast.keys())
@pytest.mark.fast
def test_Get_country2_fast(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data_slow.items(), ids=data_slow.keys())
@pytest.mark.slow
def test_Get_country2_slow(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", test_historic.items(), ids=test_historic.keys())
@pytest.mark.slow
def test_historic_places(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected



# -*- coding: utf-8 -*-
"""
This module provides functions to identify and convert English time-related
expressions (such as years, decades, centuries, and millennia) into their
Arabic equivalents.

It includes regular expressions for matching time expressions in both English
and Arabic, and a conversion function to translate English expressions.
"""

import functools
import re

century_millennium_regex = r"(\d+)(?:st|nd|rd|th)(?:[−–\- ])(century|millennium)\s*(BCE|BC)?"
decade_regex = r"(\d{1,4})s\s*(BCE|BC)?"

REG_YEAR_EN = re.compile(
    r"\b"
    r"(?:January|February|March|April|May|June|July|August|September|October|November|December)?\s*"
    r"("
    r"\d+[−–\-]\d+"
    rf"|{decade_regex}"
    r"|\d{1,4}\s*(?:BCE|BC)?"
    r")"
    r"\b",
    re.I,
)
REG_CENTURY_EN = re.compile(rf"\b{century_millennium_regex}\b", re.I)

REG_YEAR_AR = re.compile(
    r"\b"
    r"(?:يناير|فبراير|مارس|أبريل|مايو|يونيو|يوليو|أغسطس|سبتمبر|أكتوبر|نوفمبر|ديسمبر)?\s*"
    r"("
    r"\d+[−–\-]\d+"
    r"|عقد \d{1,4} *(?:ق\.م|ق م|قبل الميلاد)?"
    r"|(?:القرن|الألفية)? \d{1,4} *(?:ق\.م|ق م|قبل الميلاد)?"
    r")"
    r"\b",
    re.I,
)

REG_CENTURY_AR = re.compile(r"\bب*(?:القرن|الألفية) \d+ *(?:ق\.م|ق م|قبل الميلاد)?\b", re.I)

# Additional precompiled regex patterns
REG_SUB_CATEGORY = re.compile(r"^Category:", re.I)
REG_YEAR_BC_PATTERN = re.compile(r"^(\d+)\s*(BCE|BC)$", re.I)
REG_YEAR_RANGE_PATTERN = re.compile(r"^\d+[−–\-]\d+$", re.I)

# Month-related patterns
MONTH_STR = "|".join(
    [
        "january",
        "february",
        "march",
        "april",
        "may",
        "june",
        "july",
        "august",
        "september",
        "october",
        "november",
        "december",
    ]
)
REG_MONTH_YEAR = re.compile(rf"^({MONTH_STR})\s*(\d+)\s*$", re.I)
REG_MONTH_YEAR_BC = re.compile(rf"^({MONTH_STR})\s*" + r"(\d{1,4})\s*(BCE|BC)$", re.I)
REG_DECADE = re.compile(rf"^{decade_regex}$", re.I)
REG_CENTURY_MILLENNIUM = re.compile(rf"^{century_millennium_regex}$", re.I)

# --- Numeric range ---


def expand_range(year_text: str) -> str:
    """Expand shorthand year ranges like ``1990-92`` into full spans."""
    parts = year_text.split("-")
    if len(parts) == 1:
        return year_text
    try:
        first = int(parts[0].rstrip("s"))
        second = parts[1].rstrip("s")
        if len(second) == 2:
            prefix = str(first)[: len(str(first)) - 2]
            second = int(prefix + second)
        else:
            second = int(second)
        return f"{first}-{second}"
    except ValueError:
        return year_text


def match_time_ar(ar_value: str) -> list[str]:
    """Find Arabic year-like expressions within a string."""
    ar_matches = [m.group().strip() for m in REG_YEAR_AR.finditer(f" {ar_value} ")]
    # ar_matches.extend([m.group().strip() for m in REG_CENTURY_AR.finditer(f" {ar_value} ")])
    return ar_matches


def match_time_ar_first(ar_key: str) -> str:
    """Return the first English time match or an empty string."""
    ar_matches = match_time_ar(ar_key)
    return ar_matches[0] if ar_matches else ""


def match_time_en(en_key: str) -> list[str]:
    """Locate English year or century expressions within a string."""
    en_key = REG_SUB_CATEGORY.sub("", en_key)
    en_matches = [m.group().strip() for m in REG_YEAR_EN.finditer(f" {en_key} ")]
    en_matches.extend([m.group().strip() for m in REG_CENTURY_EN.finditer(f" {en_key} ")])
    return en_matches


def match_time_en_first(en_key: str) -> str:
    """Return the first English time match or an empty string."""
    en_matches = match_time_en(en_key)
    return en_matches[0] if en_matches else ""


def convert_time_to_arabic(en_year: str) -> str:
    """Convert an English time expression into its Arabic equivalent."""
    en_year = en_year.strip()  # .replace("–", "-").replace("−", "-")

    if en_year.lower().startswith("the "):
        en_year = en_year[4:]

    month_map = {
        "january": "يناير",
        "february": "فبراير",
        "march": "مارس",
        "april": "أبريل",
        "may": "مايو",
        "june": "يونيو",
        "july": "يوليو",
        "august": "أغسطس",
        "september": "سبتمبر",
        "october": "أكتوبر",
        "november": "نوفمبر",
        "december": "ديسمبر",
    }
    if en_year.isdigit():
        return en_year

    # --- Month ---
    if month_map.get(en_year.lower()):
        return month_map[en_year.lower()]

    # --- Month + Year ---
    m = REG_MONTH_YEAR.match(en_year)
    if m:
        month = month_map[m.group(1).lower()]
        result = f"{month} {m.group(2)}"
        return result

    # --- Month + Year + BC ---
    m = REG_MONTH_YEAR_BC.match(en_year)
    if m:
        month = month_map[m.group(1).lower()]
        bc = " ق م"
        result = f"{month} {m.group(2)}{bc}"
        return result

    # --- Year + BC ---
    m = REG_YEAR_BC_PATTERN.match(en_year)
    if m:
        result = f"{m.group(1)} ق م"
        return result

    # --- Decade (with optional BC/BCE) ---
    m = REG_DECADE.match(en_year)
    if m:
        bc = " ق م" if m.group(2) else ""
        result = f"عقد {m.group(1)}{bc}"
        return result

    # --- Century/Millennium ---
    m = REG_CENTURY_MILLENNIUM.match(en_year)
    if m:
        num = int(m.group(1))
        bc = " ق م" if m.group(3) else ""
        ty = "القرن" if m.group(2) == "century" else "الألفية"
        result = f"{ty} {num}{bc}"
        return result

    if REG_YEAR_RANGE_PATTERN.match(en_year):
        # --- (no expansion wanted) ---
        # return expand_range(en_year)
        return en_year

    # --- Fallback ---
    return ""


@functools.lru_cache(maxsize=10000)
def match_en_return_ar(category: str) -> dict[str, str]:
    """Convert an English time expression into its Arabic equivalent."""
    en_years = match_time_en(category)
    data = {year: convert_time_to_arabic(year) for year in en_years}
    return data


__all__ = [
    "match_time_ar",
    "match_time_en",
    "match_time_en_first",
    "match_time_ar_first",
    "convert_time_to_arabic",
    "match_en_return_ar",
]



#!/usr/bin/python3
"""
!
"""

import functools
import re

from ...helps import logger
from ...translations import RELIGIOUS_KEYS_PP, jobs_mens_data, jobs_womens_data
from ...translations_formats import MultiDataFormatterBase, format_multi_data

REGEX_WOMENS = re.compile(r"\b(womens|women)\b", re.I)

NAT_BEFORE_OCC_BASE = [
    "murdered abroad",
    "contemporary",
    "tour de france stage winners",
    "deafblind",
    "deaf",
    "blind",
    "jews",
    # "women's rights activists",
    "human rights activists",
    "imprisoned",
    "imprisoned abroad",
    "conservationists",
    "expatriate",
    "defectors",
    "scholars of islam",
    "scholars-of-islam",
    "amputees",
    "executed abroad",
    "executed",
    "emigrants",
]


@functools.lru_cache(maxsize=1)
def _load_womens_bot() -> MultiDataFormatterBase:
    religions_data = {x: v["females"] for x, v in RELIGIOUS_KEYS_PP.items() if v.get("females")}

    female_formatted_data = {
        "female {job_en}": "{job_ar}",
        "people female {rele_en}": "{rele_ar}",
        "female {rele_en}": "{rele_ar}",
        "female {rele_en} {job_en}": "{job_ar} {rele_ar}",
        "female {job_en} {rele_en}": "{job_ar} {rele_ar}",
        "{rele_en} female {job_en}": "{job_ar} {rele_ar}",
        "{job_en} female {rele_en}": "{job_ar} {rele_ar}",
        "{rele_en} female saints": "قديسات {rele_ar}",
        "{rele_en} female eugenicists": "عالمات {rele_ar} متخصصات في تحسين النسل",
        "{rele_en} female politicians who committed suicide": "سياسيات {rele_ar} أقدمن على الانتحار",
        "{rele_en} female contemporary artists": "فنانات {rele_ar} معاصرات",
    }

    return format_multi_data(
        formatted_data=female_formatted_data,
        data_list=religions_data,
        key_placeholder="{rele_en}",
        value_placeholder="{rele_ar}",
        data_list2=jobs_womens_data,
        key2_placeholder="{job_en}",
        value2_placeholder="{job_ar}",
        text_after="",
        search_first_part=True,
    )


@functools.lru_cache(maxsize=1)
def _load_mens_bot() -> MultiDataFormatterBase:
    religions_data = {x: v["males"] for x, v in RELIGIOUS_KEYS_PP.items() if v.get("males")}

    formatted_data = {
        "people {job_en}": "{job_ar}",
        "{job_en}": "{job_ar}",
        "people {rele_en}": "{rele_ar}",
        "{rele_en}": "{rele_ar}",
        "{rele_en} expatriate": "{rele_ar} مغتربون",
        # "{rele_en} {job_en}": "{job_ar} {rele_ar}",
        # "{job_en} {rele_en}": "{job_ar} {rele_ar}",
        "{rele_en} {job_en}": "{job_ar} {rele_ar}",
        "{job_en} {rele_en}": "{job_ar} {rele_ar}",
        "male {job_en}": "{job_ar} ذكور",
        "male {rele_en}": "{rele_ar} ذكور",
        "{rele_en} male {job_en}": "{job_ar} ذكور {rele_ar}",
        "{job_en} male {rele_en}": "{job_ar} ذكور {rele_ar}",
        "fictional {rele_en} religious workers": "عمال دينيون {rele_ar} خياليون",
        "{rele_en} religious workers": "عمال دينيون {rele_ar}",
        "{rele_en} emigrants": "{rele_ar} مهاجرون",
        "{rele_en} saints": "قديسون {rele_ar}",
        "{rele_en} eugenicists": "علماء {rele_ar} متخصصون في تحسين النسل",
        "{rele_en} politicians who committed suicide": "سياسيون {rele_ar} أقدموا على الانتحار",
        "{rele_en} contemporary artists": "فنانون {rele_ar} معاصرون",
        # TODO: ADD DATA FROM NAT_BEFORE_OCC_BASE
        "{rele_en} scholars of islam": "{rele_ar} باحثون عن الإسلام",
        "{rele_en} female rights activists": "{rele_ar} ناشطون في حقوق المرأة",
    }

    for x in NAT_BEFORE_OCC_BASE:
        if jobs_mens_data.get(x):
            formatted_data[f"{{rele_en}} {x}"] = f"{{rele_ar}} {jobs_mens_data[x]}"
            formatted_data[f"{x} {{rele_en}}"] = f"{{rele_ar}} {jobs_mens_data[x]}"

    jobs_data = dict(jobs_mens_data)
    jobs_data.update(
        {
            "philosophers and theologians": "فلاسفة ولاهوتيون",
        }
    )

    return format_multi_data(
        formatted_data=formatted_data,
        data_list=religions_data,
        key_placeholder="{rele_en}",
        value_placeholder="{rele_ar}",
        data_list2=jobs_data,
        key2_placeholder="{job_en}",
        value2_placeholder="{job_ar}",
        search_first_part=True,
    )


@functools.lru_cache(maxsize=None)
def womens_result(category: str) -> str:
    """ """
    logger.debug(f"\t xx start: <<lightred>>womens_result >> <<lightpurple>> {category=}")

    nat_bot = _load_womens_bot()
    return nat_bot.search_all_category(category)


@functools.lru_cache(maxsize=None)
def mens_result(category: str) -> str:
    """ """
    logger.debug(f"\t xx start: <<lightred>>mens_result >> <<lightpurple>> {category=}")

    nat_bot = _load_mens_bot()
    return nat_bot.search_all_category(category)


def fix_keys(category: str) -> str:
    category = category.replace("'", "").lower()

    replacements = {
        "expatriates": "expatriate",
    }

    for old, new in replacements.items():
        category = category.replace(old, new)

    category = REGEX_WOMENS.sub("female", category)
    return category.strip()


@functools.lru_cache(maxsize=None)
def new_religions_jobs_with_suffix(category: str) -> str:
    """
    Resolves jobs for religious groups, handling both male and female variations.
    This function attempts to find a translation for a given category by checking
    against male-specific job/religion combinations first, and then female-specific
    combinations if no male-specific match is found.
    Args:
        category: The category string to translate.
    Returns:
        The translated Arabic category string, or an empty string if no match is found.
    """
    category = fix_keys(category)
    logger.debug(f"\t xx start: <<lightred>>new_religions_jobs_with_suffix >> <<lightpurple>> {category=}")

    return mens_result(category) or womens_result(category)


__all__ = [
    "new_religions_jobs_with_suffix",
    "mens_result",
    "womens_result",
]



"""
Tests
"""

import pytest

from ArWikiCats.new_resolvers.resolve_languages import add_definite_article
from ArWikiCats.new_resolvers.resolve_languages import resolve_languages_labels as resolve_languages_labels
from ArWikiCats.translations import language_key_translations

languages_key_subset = {k: language_key_translations[k] for k in list(language_key_translations.keys())[:15]}

data = {
    "abkhazian languages grammar": "قواعد اللغات الأبخازية",
    "abkhazian-language grammar": "قواعد اللغة الأبخازية",
    "arabic-language grammar": "قواعد اللغة العربية",
    "arabic languages grammar": "قواعد اللغات العربية",
    "pali-language grammar": "قواعد اللغة البالية",
    "pali languages grammar": "قواعد اللغات البالية",
    "balinese-language grammar": "قواعد اللغة البالية",
    "balinese languages grammar": "قواعد اللغات البالية",
    "afrikaans-language grammar": "قواعد اللغة الإفريقية",
    "afrikaans languages grammar": "قواعد اللغات الإفريقية",
    "afar-language grammar": "قواعد اللغة العفارية",
    "afar languages grammar": "قواعد اللغات العفارية",
    "abkhazian films": "أفلام باللغة الأبخازية",
    "abkhazian language": "لغة أبخازية",
    "abkhazian languages dialects": "لهجات اللغات الأبخازية",
    "abkhazian languages films": "أفلام باللغات الأبخازية",
    "abkhazian languages given names": "أسماء شخصية باللغات الأبخازية",
    "abkhazian languages surnames": "ألقاب باللغات الأبخازية",
    "abkhazian languages writing system": "نظام كتابة اللغات الأبخازية",
    "abkhazian languages": "اللغات الأبخازية",
    "abkhazian-language dialects": "لهجات اللغة الأبخازية",
    "abkhazian-language given names": "أسماء شخصية باللغة الأبخازية",
    "abkhazian-language surnames": "ألقاب باللغة الأبخازية",
    "abkhazian-language writing system": "نظام كتابة اللغة الأبخازية",
    "abkhazian-language": "اللغة الأبخازية",
    "afar films": "أفلام باللغة العفارية",
    "afar language": "لغة عفارية",
    "afar languages dialects": "لهجات اللغات العفارية",
    "afar languages films": "أفلام باللغات العفارية",
    "afar languages given names": "أسماء شخصية باللغات العفارية",
    "afar languages surnames": "ألقاب باللغات العفارية",
    "afar languages writing system": "نظام كتابة اللغات العفارية",
    "afar languages": "اللغات العفارية",
    "afar-language dialects": "لهجات اللغة العفارية",
    "afar-language given names": "أسماء شخصية باللغة العفارية",
    "afar-language surnames": "ألقاب باللغة العفارية",
    "afar-language writing system": "نظام كتابة اللغة العفارية",
    "afar-language": "اللغة العفارية",
    "afrikaans films": "أفلام باللغة الإفريقية",
    "afrikaans language": "لغة إفريقية",
    "afrikaans languages dialects": "لهجات اللغات الإفريقية",
    "afrikaans languages films": "أفلام باللغات الإفريقية",
    "afrikaans languages given names": "أسماء شخصية باللغات الإفريقية",
    "afrikaans languages surnames": "ألقاب باللغات الإفريقية",
    "afrikaans languages writing system": "نظام كتابة اللغات الإفريقية",
    "afrikaans languages": "اللغات الإفريقية",
    "afrikaans-language dialects": "لهجات اللغة الإفريقية",
    "afrikaans-language given names": "أسماء شخصية باللغة الإفريقية",
    "afrikaans-language surnames": "ألقاب باللغة الإفريقية",
    "afrikaans-language writing system": "نظام كتابة اللغة الإفريقية",
    "afrikaans-language": "اللغة الإفريقية",
    "balinese films": "أفلام باللغة البالية",
    "balinese language": "لغة بالية",
    "balinese languages dialects": "لهجات اللغات البالية",
    "balinese languages films": "أفلام باللغات البالية",
    "balinese languages given names": "أسماء شخصية باللغات البالية",
    "balinese languages surnames": "ألقاب باللغات البالية",
    "balinese languages writing system": "نظام كتابة اللغات البالية",
    "balinese languages": "اللغات البالية",
    "balinese-language dialects": "لهجات اللغة البالية",
    "balinese-language given names": "أسماء شخصية باللغة البالية",
    "balinese-language surnames": "ألقاب باللغة البالية",
    "balinese-language writing system": "نظام كتابة اللغة البالية",
    "balinese-language": "اللغة البالية",
    "english language": "لغة إنجليزية",
    "pali films": "أفلام باللغة البالية",
    "pali language": "لغة بالية",
    "pali languages dialects": "لهجات اللغات البالية",
    "pali languages films": "أفلام باللغات البالية",
    "pali languages given names": "أسماء شخصية باللغات البالية",
    "pali languages surnames": "ألقاب باللغات البالية",
    "pali languages writing system": "نظام كتابة اللغات البالية",
    "pali languages": "اللغات البالية",
    "pali-language dialects": "لهجات اللغة البالية",
    "pali-language given names": "أسماء شخصية باللغة البالية",
    "pali-language surnames": "ألقاب باللغة البالية",
    "pali-language writing system": "نظام كتابة اللغة البالية",
    "pali-language": "اللغة البالية",
    "arabic films": "أفلام باللغة العربية",
    "arabic language": "لغة عربية",
    "arabic languages dialects": "لهجات اللغات العربية",
    "arabic languages films": "أفلام باللغات العربية",
    "arabic languages given names": "أسماء شخصية باللغات العربية",
    "arabic languages surnames": "ألقاب باللغات العربية",
    "arabic languages writing system": "نظام كتابة اللغات العربية",
    "arabic languages": "اللغات العربية",
    "arabic-language dialects": "لهجات اللغة العربية",
    "arabic-language given names": "أسماء شخصية باللغة العربية",
    "arabic-language surnames": "ألقاب باللغة العربية",
    "arabic-language writing system": "نظام كتابة اللغة العربية",
    "arabic-language": "اللغة العربية",
}


@pytest.mark.parametrize("category, expected", data.items(), ids=data.keys())
def test_Lang_work_main(category: str, expected: str) -> None:
    result = resolve_languages_labels(category)
    assert result == expected


def test_lang_work() -> None:
    # Test with a basic input
    result = resolve_languages_labels("test language")
    assert isinstance(result, str)

    result_empty = resolve_languages_labels("")
    assert isinstance(result_empty, str)

    # Test with various inputs
    result_various = resolve_languages_labels("english language")
    assert isinstance(result_various, str)


# -----------------------------------------------------------
# 1) Parametrize: test direct keys from languages_key_subset
# -----------------------------------------------------------


@pytest.mark.parametrize(
    "key, expected",
    [(k, v) for k, v in languages_key_subset.items()],
    ids=list(languages_key_subset.keys()),
)
def test_lang_work_direct(key: str, expected: str) -> None:
    """Test resolve_languages_labels for direct language keys."""
    result = resolve_languages_labels(key)
    # resolve_languages_labels may return full label or variant formatting
    assert isinstance(result, str)
    # If expected is Arabic label, result must equal or include it
    if result:
        assert expected in result or result == expected


# -----------------------------------------------------------
# 2) Parametrize: test "<key> language"
# -----------------------------------------------------------
data_2 = [(k, f"لغة {languages_key_subset[k]}") for k in languages_key_subset if not k.endswith(" language")]


@pytest.mark.parametrize(
    "key, expected",
    data_2,
    ids=[x[0] for x in data_2],
)
def test_lang_work_language_suffix(key: str, expected: str) -> None:
    """Test '<lang> language' format."""
    candidate = f"{key} language"
    result = resolve_languages_labels(candidate)

    if candidate in languages_key_subset:
        # Must exactly match mapping
        assert result == languages_key_subset[candidate]
    else:
        # If our synthesized key does not exist, result may be empty or valid
        assert result is not None


# -----------------------------------------------------------
# 3) Parametrize: test "<key> films"
# -----------------------------------------------------------
@pytest.mark.parametrize(
    "key, arabic",
    [(k, v) for k, v in languages_key_subset.items()],
    ids=[k for k in languages_key_subset.keys()],
)
def test_lang_work_films_suffix(key: str, arabic: str) -> None:
    """Test '<lang> films' -> 'أفلام ب<ArabicLabel>'."""
    base = key.replace("-language", "")
    candidate = f"{base} films"

    result = resolve_languages_labels(candidate)

    if result:
        assert isinstance(result, str)
        assert "أفلام ب" in result
        assert add_definite_article(arabic) in result


# -----------------------------------------------------------
# 5) Parametrize: key + topic suffix such as 'grammar', 'writing system', etc.
# -----------------------------------------------------------
TOPIC_SUFFIXES = [
    "grammar",
    "writing system",
    "dialects",
    "surnames",
    "given names",
]

data_x = [(k, suf) for k in languages_key_subset for suf in TOPIC_SUFFIXES]


@pytest.mark.parametrize(
    "key, suffix",
    data_x,
    ids=[f"{x[0]}-{x[1]}" for x in data_x],
)
def test_lang_work_topics(key: str, suffix: str) -> None:
    """Test '<lang> grammar', '<lang> writing system', etc."""
    candidate = f"{key} {suffix}"
    result = resolve_languages_labels(candidate)

    assert result is not None
    if result:
        # Must contain Arabic name at least
        assert add_definite_article(languages_key_subset[key]) in result



#!/usr/bin/python3
"""

Resolves category labels for sports federations based on nationality.
This module constructs a formatter that combines nationality data with sports data
to translate category titles like "{nationality} {sport} federation" into Arabic.

NOTE: compare it with ArWikiCats/new_resolvers/sports_formats_teams/sport_lab_nat.py

"""
import functools

from ...helps import logger
from ...new.handle_suffixes import resolve_sport_category_suffix_with_mapping
from ...translations.nats.Nationality import all_country_with_nat_ar
from ...translations.sports.Sport_key import SPORT_KEY_RECORDS
from ...translations_formats import MultiDataFormatterBaseV2, format_multi_data_v2
from ..nationalities_resolvers.data import sports_formatted_data_for_jobs
from .utils import fix_keys


@functools.lru_cache(maxsize=1)
def _get_sorted_teams_labels() -> dict[str, str]:
    teams_label_mappings_ends_old = {
        "champions": "أبطال",
        "events": "أحداث",
        "films": "أفلام",
        "finals": "نهائيات",
        "home stadiums": "ملاعب",
        "lists": "قوائم",
        "manager history": "تاريخ مدربو",
        "managers": "مدربو",
        "matches": "مباريات",
        "navigational boxes": "صناديق تصفح",
        "non-profit organizations": "منظمات غير ربحية",
        "non-profit publishers": "ناشرون غير ربحيون",
        "organisations": "منظمات",
        "organizations": "منظمات",
        "players": "لاعبو",
        "positions": "مراكز",
        "records and statistics": "سجلات وإحصائيات",
        "records": "سجلات",
        "results": "نتائج",
        "rivalries": "دربيات",
        "scouts": "كشافة",
        "squads": "تشكيلات",
        "statistics": "إحصائيات",
        "templates": "قوالب",
        "trainers": "مدربو",
        "umpires": "حكام",
        "venues": "ملاعب",
    }

    mappings_data = {
        "tournaments": "بطولات",
        "leagues": "دوريات",
        "coaches": "مدربو",
        "clubs": "أندية",
        "clubs and teams": "أندية وفرق",
        "competitions": "منافسات",
        "chairmen and investors": "رؤساء ومسيرو",
        "cups": "كؤوس",
    }

    mappings_data = dict(
        sorted(
            mappings_data.items(),
            key=lambda k: (-k[0].count(" "), -len(k[0])),
        )
    )
    return mappings_data


def _levels_data() -> dict[str, str]:
    data = {
        "{en} league of {en_sport}": "الدوري {the_male} {sport_team}",
        "{en} {en_sport} premier league": "الدوري {the_male} الممتاز {sport_team}",
        "{en} premier {en_sport} league": "الدوري {the_male} الممتاز {sport_team}",
        # "bangladesh football premier leagues": "تصنيف:دوريات كرة قدم بنغلاديشية من الدرجة الممتازة",
        "{en} {en_sport} premier": "{sport_jobs} {female} من الدرجة الممتازة",
        "{en_sport} premier": "{sport_jobs} من الدرجة الممتازة",
        "{en} premier {en_sport}": "{sport_jobs} {female} من الدرجة الممتازة",
        "premier {en_sport}": "{sport_jobs} من الدرجة الممتازة",
    }
    LEVELS: dict[str, str] = {
        "premier": "الدرجة الممتازة",
        "top level": "الدرجة الأولى",
        "first level": "الدرجة الأولى",
        "first tier": "الدرجة الأولى",
        "second level": "الدرجة الثانية",
        "second tier": "الدرجة الثانية",
        "third level": "الدرجة الثالثة",
        "third tier": "الدرجة الثالثة",
        "fourth level": "الدرجة الرابعة",
        "fourth tier": "الدرجة الرابعة",
        "fifth level": "الدرجة الخامسة",
        "fifth tier": "الدرجة الخامسة",
        "sixth level": "الدرجة السادسة",
        "sixth tier": "الدرجة السادسة",
        "seventh level": "الدرجة السابعة",
        "seventh tier": "الدرجة السابعة",
    }
    for level, lvl_lab in LEVELS.items():
        data.update(
            {
                f"{{en}} {{en_sport}} {level} leagues": f"دوريات {{sport_jobs}} {{female}} من {lvl_lab}",
                f"{{en}} {level} {{en_sport}} leagues": f"دوريات {{sport_jobs}} {{female}} من {lvl_lab}",
                f"national {{en_sport}} {level} leagues": f"دوريات {{sport_jobs}} وطنية من {lvl_lab}",
                f"{{en_sport}} {level} leagues": f"دوريات {{sport_jobs}} من {lvl_lab}",
                f"{level} {{en_sport}} leagues": f"دوريات {{sport_jobs}} من {lvl_lab}",
            }
        )

    return data


@functools.lru_cache(maxsize=1)
def _load_sports_formatted_data() -> dict[str, str]:
    sports_formatted_data = {
        "{en} league cup": "كأس الدوري {the_male}",
        "{en} independence cup": "كأس الاستقلال {the_male}",
        "amateur {en_sport} world cup": "كأس العالم {sport_team} للهواة",
        "mens {en_sport} world cup": "كأس العالم {sport_team} للرجال",
        "womens {en_sport} world cup": "كأس العالم {sport_team} للسيدات",
        "{en_sport} world cup": "كأس العالم {sport_team}",
        "youth {en_sport} world cup": "كأس العالم {sport_team} للشباب",
        "olympic gold medalists in {en_sport}": "فائزون بميداليات ذهبية أولمبية في {sport_label}",
        "olympic silver medalists in {en_sport}": "فائزون بميداليات فضية أولمبية في {sport_label}",
        "olympic bronze medalists in {en_sport}": "فائزون بميداليات برونزية أولمبية في {sport_label}",
        "{en} mens {en_sport} national team": "منتخب {ar} {sport_team} للرجال",
        "{en} mens u23 national {en_sport} team": "منتخب {ar} {sport_team} تحت 23 سنة للرجال",
        "{en} {en_sport} national team": "منتخب {ar} {sport_team}",
        "{en} open ({en_sport})": "{ar} المفتوحة {sport_team}",
        "{en} open {en_sport}": "{ar} المفتوحة {sport_team}",
        "first league of {en}": "دوري {ar} الممتاز",
        "{en}-american coaches of canadian-football": "مدربو كرة قدم كندية أمريكيون {males}",
        # "yemeni men's basketball players" : "لاعبو كرة سلة رجالية يمنيون",
        "{en} mens {en_sport} players": "لاعبو {sport_jobs} {males}",
        # american coaches of basketball
        "{en} coaches of {en_sport}": "مدربو {sport_jobs} {males}",
        "{en}-american coaches of {en_sport}": "مدربو {sport_jobs} أمريكيون {males}",
        # coaches of basketball
        "coaches of {en_sport}": "مدربو {sport_jobs}",
        "players of {en_sport}": "لاعبو {sport_jobs}",
        # lithuanian expatriate basketball people "أعلام كرة سلة ليتوانيون مغتربون"
        "{en} expatriate {en_sport} peoplee": "أعلام {sport_jobs} {males} مغتربون",
        "{en} expatriate {en_sport} people": "أعلام {sport_jobs} {males} مغتربون",
        # expatriate basketball people
        "expatriate {en_sport} peoplee": "أعلام {sport_jobs} مغتربون",
        "expatriate {en_sport} people": "أعلام {sport_jobs} مغتربون",
        "{en} {en_sport} people": "أعلام {sport_label} {the_female}",
        # _build_new_kkk() -> dict[str, str]:
        # Category:National junior womens goalball teams
        "{en} national junior mens {en_sport} team": "منتخب {ar} {sport_team} للناشئين",
        "{en} national junior {en_sport} team": "منتخب {ar} {sport_team} للناشئين",
        "{en} national womens {en_sport} team": "منتخب {ar} {sport_team} للسيدات",
        "{en} national mens {en_sport} team": "منتخب {ar} {sport_team} للرجال",
        # _build_male_nat()
        "{en} league": "الدوري {the_male}",
        "{en} fa cup": "كأس الاتحاد {the_male}",
        "{en} {en_sport} super league": "دوري السوبر {sport_label} {the_male}",
        "{en} professional {en_sport} league": "دوري {sport_label} {the_male} للمحترفين",
        # Middle East Rally Championship بطولة الشرق الأوسط للراليات
        "{en} {en_sport} league": "الدوري {the_male} {sport_team}",
        "{en} {en_sport} league administrators": "مدراء الدوري {the_male} {sport_team}",
        "{en} {en_sport} league players": "لاعبو الدوري {the_male} {sport_team}",
        "{en} {en_sport} league playerss": "لاعبو الدوري {the_male} {sport_team}",
        # tab[Category:American Indoor Soccer League coaches] = "تصنيف:مدربو الدوري الأمريكي لكرة القدم داخل الصالات"
        "{en} indoor {en_sport} league": "الدوري {the_male} {sport_team} داخل الصالات",
        "{en} outdoor {en_sport} league": "الدوري {the_male} {sport_team} في الهواء الطلق",
        # tab[Category:Canadian Major Indoor Soccer League seasons] = "تصنيف:مواسم الدوري الرئيسي الكندي لكرة القدم داخل الصالات"
        "{en} major indoor {en_sport} league": "الدوري الرئيسي {the_male} {sport_team} داخل الصالات",
        # "yemeni major indoor wheelchair football league": "الدوري الرئيسي اليمني لكرة القدم على الكراسي المتحركة داخل الصالات",
        "{en} major indoor wheelchair {en_sport} league": "الدوري الرئيسي {the_male} {sport_team} على الكراسي المتحركة داخل الصالات",
        # ---
        # SPORT_FORMATS_FEMALE_NAT
        # [chinese outdoor boxing] : "تصنيف:بوكسينغ صينية في الهواء الطلق",
        "{en} outdoor {en_sport}": "{sport_jobs} {female} في الهواء الطلق",
        # [Category:American Indoor Soccer] : "تصنيف:كرة قدم أمريكية داخل الصالات",
        "{en} indoor {en_sport}": "{sport_jobs} {female} داخل الصالات",
        # data
        # "Category:zaïrean wheelchair sports federation": "تصنيف:الاتحاد الزائيري للرياضة على الكراسي المتحركة",
        # "Category:surinamese sports federation": "تصنيف:الاتحاد السورينامي للرياضة",
        "{en} sports federation": "الاتحاد {the_male} للرياضة",
        "{en} wheelchair sports federation": "الاتحاد {the_male} للرياضة على الكراسي المتحركة",
        "{en} wheelchair racers": "متسابقو كراسي متحركة {males}",
        "{en} mens wheelchair racers": "متسابقو كراسي متحركة {males}",
        "{en} {en_sport} federation": "الاتحاد {the_male} {sport_team}",
        "ladies {en} {en_sport} championships": "بطولة {ar} {sport_team} للسيدات",
        "ladies {en} {en_sport} tour": "بطولة {ar} {sport_team} للسيدات",
        "womens {en} {en_sport} tour": "بطولة {ar} {sport_team} للسيدات",
        "{en} {en_sport} championships": "بطولة {ar} {sport_team}",
        "{en} {en_sport} championshipszz": "بطولة {ar} {sport_team}",
        "{en} {en_sport} tour": "بطولة {ar} {sport_team}",
        # Category:yemeni Women's Football League
        "womens {en} {en_sport} league": "الدوري {the_male} {sport_team} للسيدات",
        "womens {en} {en_sport} league players": "لاعبات الدوري {the_male} {sport_team} للسيدات",
        "{en} womens {en_sport} league": "الدوري {the_male} {sport_team} للسيدات",
        "{en} womens {en_sport} league players": "لاعبات الدوري {the_male} {sport_team} للسيدات",
        "womens national {en_sport} league": "الدوري الوطني {sport_team} للسيدات",
        "{en} national {en_sport} teams": "منتخبات {sport_jobs} وطنية {female}",
        "{en} womens {en_sport} players": "لاعبات {sport_jobs} {females}",
        "womens {en_sport} players": "لاعبات {sport_jobs}",
        "{en} womens {en_sport} playerss": "لاعبات {sport_jobs} {females}",
        "womens {en_sport} playerss": "لاعبات {sport_jobs}",
        "{en} womens national {en_sport} team": "منتخب {ar} {sport_team} للسيدات",
        "{en} womens national {en_sport} team players": "لاعبات منتخب {ar} {sport_team} للسيدات",
        "{en} national {en_sport} team": "منتخب {ar} {sport_team}",
        "{en} national {en_sport} team players": "لاعبو منتخب {ar} {sport_team}",
        "{en} womens international footballers": "لاعبات منتخب {ar} لكرة القدم للسيدات",
        "{en} womens youth international footballers": "لاعبات منتخب {ar} لكرة القدم للشابات",
        "{en} womens international {en_sport} players": "لاعبات {sport_jobs} دوليات من {ar}",
        "{en} international footballers": "لاعبو منتخب {ar} لكرة القدم",
        "{en} international {en_sport} players": "لاعبو {sport_jobs} دوليون من {ar}",
        "{en} {en_sport} association": "الرابطة {the_female} {sport_team}",
        "womens {en} {en_sport} association": "الرابطة {the_female} {sport_team} للسيدات",
        # Category:African women's national association football teams
        "womens national {en_sport} teams": "منتخبات {sport_jobs} وطنية للسيدات",
        "{en} womens national {en_sport} teams": "منتخبات {sport_jobs} وطنية {female} للسيدات",
        "{en} womens {en_sport}": "{sport_jobs} {female} للسيدات",
        # northern ireland national men's football teams
        "national mens {en_sport} teams": "منتخبات {sport_jobs} وطنية للرجال",
        "{en} national mens {en_sport} teams": "منتخبات {sport_jobs} وطنية {female} للرجال",
        # NAT_P17_OIOI data
        "{en} amateur {en_sport} championship": "بطولة {ar} {sport_team} للهواة",
        "{en} amateur {en_sport} championships": "بطولة {ar} {sport_team} للهواة",
        "{en} championships ({en_sport})": "بطولة {ar} {sport_team}",
        "{en} championships {en_sport}": "بطولة {ar} {sport_team}",
        "{en} mens {en_sport} championship": "بطولة {ar} {sport_team} للرجال",
        "{en} mens {en_sport} championships": "بطولة {ar} {sport_team} للرجال",
        "{en} {en_sport} championship": "بطولة {ar} {sport_team}",
        "{en} {en_sport} indoor championship": "بطولة {ar} {sport_team} داخل الصالات",
        "{en} {en_sport} indoor championships": "بطولة {ar} {sport_team} داخل الصالات",
        "{en} {en_sport} junior championships": "بطولة {ar} {sport_team} للناشئين",
        "{en} {en_sport} u-13 championships": "بطولة {ar} {sport_team} تحت 13 سنة",
        "{en} {en_sport} u-14 championships": "بطولة {ar} {sport_team} تحت 14 سنة",
        "{en} {en_sport} u-15 championships": "بطولة {ar} {sport_team} تحت 15 سنة",
        "{en} {en_sport} u-16 championships": "بطولة {ar} {sport_team} تحت 16 سنة",
        "{en} {en_sport} u-17 championships": "بطولة {ar} {sport_team} تحت 17 سنة",
        "{en} {en_sport} u-18 championships": "بطولة {ar} {sport_team} تحت 18 سنة",
        "{en} {en_sport} u-19 championships": "بطولة {ar} {sport_team} تحت 19 سنة",
        "{en} {en_sport} u-20 championships": "بطولة {ar} {sport_team} تحت 20 سنة",
        "{en} {en_sport} u-21 championships": "بطولة {ar} {sport_team} تحت 21 سنة",
        "{en} {en_sport} u-23 championships": "بطولة {ar} {sport_team} تحت 23 سنة",
        "{en} {en_sport} u-24 championships": "بطولة {ar} {sport_team} تحت 24 سنة",
        "{en} {en_sport} u13 championships": "بطولة {ar} {sport_team} تحت 13 سنة",
        "{en} {en_sport} u14 championships": "بطولة {ar} {sport_team} تحت 14 سنة",
        "{en} {en_sport} u15 championships": "بطولة {ar} {sport_team} تحت 15 سنة",
        "{en} {en_sport} u16 championships": "بطولة {ar} {sport_team} تحت 16 سنة",
        "{en} {en_sport} u17 championships": "بطولة {ar} {sport_team} تحت 17 سنة",
        "{en} {en_sport} u18 championships": "بطولة {ar} {sport_team} تحت 18 سنة",
        "{en} {en_sport} u19 championships": "بطولة {ar} {sport_team} تحت 19 سنة",
        "{en} {en_sport} u20 championships": "بطولة {ar} {sport_team} تحت 20 سنة",
        "{en} {en_sport} u21 championships": "بطولة {ar} {sport_team} تحت 21 سنة",
        "{en} {en_sport} u23 championships": "بطولة {ar} {sport_team} تحت 23 سنة",
        "{en} {en_sport} u24 championships": "بطولة {ar} {sport_team} تحت 24 سنة",
        "{en} outdoor {en_sport} championship": "بطولة {ar} {sport_team} في الهواء الطلق",
        "{en} outdoor {en_sport} championships": "بطولة {ar} {sport_team} في الهواء الطلق",
        "{en} womens {en_sport} championship": "بطولة {ar} {sport_team} للسيدات",
        "{en} womens {en_sport} championships": "بطولة {ar} {sport_team} للسيدات",
        "{en} youth {en_sport} championship": "بطولة {ar} {sport_team} للشباب",
        "{en} youth {en_sport} championships": "بطولة {ar} {sport_team} للشباب",
        "{en} mens {en_sport} cup": "كأس {ar} {sport_team} للرجال",
        "{en} womens {en_sport} cup": "كأس {ar} {sport_team} للسيدات",
        "{en} {en_sport} cup": "كأس {ar} {sport_team}",
        # NAT_P17_OIOI_TO_CHECK data
        # "yemeni defunct basketball cup": "كؤوس كرة سلة يمنية سابقة",
        "{en} defunct {en_sport} cup": "كؤوس {sport_jobs} {female} سابقة",
        "{en} domestic {en_sport} cup": "كؤوس {sport_jobs} {female} محلية",
        # "yemeni domestic basketball": "كرة سلة يمنية محلية",
        "{en} domestic {en_sport}": "{sport_jobs} {female} محلية",
        "{en} domestic womens {en_sport}": "{sport_jobs} {female} محلية للسيدات",
        # "yemeni football": "كرة قدم يمنية",
        # "{en} {en_sport}": "{sport_jobs} {female}",
        "{en} federation cup": "كأس الاتحاد {the_male}",
        "{en} {en_sport}": "{sport_label} {the_female}",  # Category:American_basketball
        "{en} current {en_sport} seasons": "مواسم {sport_jobs} {female} حالية",
        "{en} reserve {en_sport}": "{sport_jobs} {female} احتياطية",
        "{en} defunct indoor {en_sport}": "{sport_jobs} {female} داخل الصالات سابقة",
        "{en} defunct {en_sport}": "{sport_jobs} {female} سابقة",
        "{en} defunct outdoor {en_sport}": "{sport_jobs} {female} في الهواء الطلق سابقة",
        "{en} professional {en_sport}": "{sport_jobs} {female} للمحترفين",
        "{en} rugby union": "اتحاد الرجبي {the_male}",
        "{en} rugby league": "الدوري {the_male} للرجبي",
    }

    sports_formatted_data.update(_levels_data())
    sports_formatted_data.update(sports_formatted_data_for_jobs)

    WOMENS_NATIONAL_DATA = {
        x.replace("womens national", "national womens"): v
        for x, v in sports_formatted_data.items()
        if "womens national" in x
    }

    sports_formatted_data.update(WOMENS_NATIONAL_DATA)

    return sports_formatted_data


@functools.lru_cache(maxsize=1)
def _load_bot() -> MultiDataFormatterBaseV2:
    nats_data = {x: v for x, v in all_country_with_nat_ar.items() if v.get("ar")}  # and v.get("en")

    sports_data = {
        x: {
            "sport_label": v.get("label", ""),
            "sport_team": v.get("team", ""),
            "sport_jobs": v.get("jobs", ""),
        }
        for x, v in SPORT_KEY_RECORDS.items()
        if v.get("label")
    }
    sports_formatted_data = _load_sports_formatted_data()
    both_bot = format_multi_data_v2(
        formatted_data=sports_formatted_data,
        data_list=nats_data,
        key_placeholder="{en}",
        data_list2=sports_data,
        key2_placeholder="{en_sport}",
        text_after="",
        text_before="the ",
        search_first_part=True,
        use_other_formatted_data=True,
    )
    return both_bot


@functools.lru_cache(maxsize=10000)
def _resolve_nats_sport_multi_v2(category: str) -> str:
    logger.debug(f"<<yellow>> start _resolve_nats_sport_multi_v2: {category=}")

    both_bot = _load_bot()
    category = fix_keys(category)
    result = both_bot.search_all_category(category)

    logger.info_if_or_debug(f"<<yellow>> end _resolve_nats_sport_multi_v2: {category=}, {result=}", result)
    return result


def fix_result_callable(result: str, category: str, key: str, value: str) -> str:
    if result.startswith("لاعبو ") and "للسيدات" in result:
        result = result.replace("لاعبو ", "لاعبات ")

    if key == "teams" and "national" in category:
        result = result.replace("فرق ", "منتخبات ")

    return result


@functools.lru_cache(maxsize=10000)
def resolve_nats_sport_multi_v2(category: str) -> str:
    category = fix_keys(category)

    logger.debug(f"<<yellow>> start resolve_nats_sport_multi_v2: {category=}")
    teams_label_mappings_ends = _get_sorted_teams_labels()

    result = resolve_sport_category_suffix_with_mapping(
        category=category,
        data=teams_label_mappings_ends,
        callback=_resolve_nats_sport_multi_v2,
        fix_result_callable=fix_result_callable,
    )

    logger.info_if_or_debug(f"<<yellow>> end resolve_nats_sport_multi_v2: {category=}, {result=}", result)
    return result


__all__ = [
    "resolve_nats_sport_multi_v2",
]



import functools

from ...helps import logger
from . import (
    ministers_resolver,
    nationalities_time_v2,
    nationalities_v2,
)


@functools.lru_cache(maxsize=None)
def resolve_nationalities_main(normalized_category) -> str:
    normalized_category = normalized_category.strip().lower().replace("category:", "")

    logger.debug("--" * 20)
    logger.debug(f"<><><><><><> <<green>> Trying nationalities_resolvers resolvers for: {normalized_category=}")

    resolved_label = (
        nationalities_v2.resolve_by_nats(normalized_category)
        or nationalities_time_v2.resolve_nats_time_v2(normalized_category)
        or ministers_resolver.resolve_secretaries_labels(normalized_category)
        or ""
    )

    logger.info_if_or_debug(
        f"<<yellow>> end nationalities_resolvers: {normalized_category=}, {resolved_label=}", resolved_label
    )
    return resolved_label


__all__ = [
    "resolve_nationalities_main",
]



"""
Tests
"""

import pytest

from ArWikiCats.new_resolvers.jobs_resolvers import resolve_jobs_main

main_data = {
    "new zealand emigrants": "نيوزيلنديون مهاجرون",
    "yemeni emigrants": "يمنيون مهاجرون",
    "the caribbean": "",
    "caribbean": "",
}


@pytest.mark.parametrize("category, expected", main_data.items(), ids=main_data.keys())
@pytest.mark.fast
def test_resolve_jobs_main(category: str, expected: str) -> None:
    label = resolve_jobs_main(category)
    assert label == expected



"""
EventLab Bot - A class-based implementation to handle category labeling
"""
import functools
from typing import Tuple

from ..config import app_settings
from ..fix import fixtitle
from ..format_bots import change_cat
from ..helps import logger
from ..legacy_bots import sport_lab_suffixes, team_work, tmp_bot
from ..legacy_bots.films_and_others_bot import te_films
from ..legacy_bots.ma_bots import ye_ts_bot
from ..legacy_bots.ma_bots2.country2_label_bot import country_2_title_work
from ..legacy_bots.ma_bots.lab_seoo_bot import event_label_work
from ..legacy_bots.make_bots.bot_2018 import get_pop_All_18
from ..legacy_bots.make_bots.ends_keys import combined_suffix_mappings
from ..legacy_bots.matables_bots.table1_bot import get_KAKO
from ..legacy_bots.o_bots import parties_bot, univer
from ..legacy_bots.o_bots.peoples_resolver import work_peoples
from ..new.end_start_bots.fax2 import get_list_of_and_cat3
from ..new.end_start_bots.fax2_episodes import get_episodes
from ..new.end_start_bots.fax2_temp import get_templates_fo
from ..new_resolvers.reslove_all import new_resolvers_all
from ..new_resolvers.sports_resolvers.raw_sports import wrap_team_xo_normal_2025_with_ends
from ..time_resolvers import time_to_arabic
from ..time_resolvers.time_to_arabic import convert_time_to_arabic
from ..translations import get_from_new_p17_final, get_from_pf_keys2
from .main_utils import list_of_cat_func_foot_ballers, list_of_cat_func_new


@functools.lru_cache(maxsize=10000)
def wrap_lab_for_country2(country: str) -> str:
    """
    TODO: should be moved to functions directory.
    Retrieve laboratory information for a specified country.
    """

    country2 = country.lower().strip()

    resolved_label = (
        new_resolvers_all(country2)
        or get_from_pf_keys2(country2)
        or get_pop_All_18(country2)
        or te_films(country2)
        or sport_lab_suffixes.get_teams_new(country2)
        or parties_bot.get_parties_lab(country2)
        or team_work.Get_team_work_Club(country2)
        or univer.te_universities(country2)
        or work_peoples(country2)
        or get_KAKO(country2)
        or convert_time_to_arabic(country2)
        or get_pop_All_18(country2)
        or ""
    )
    logger.info(f'>> wrap_lab_for_country2 "{country2}": label: {resolved_label}')

    return resolved_label


class EventLabResolver:
    """
    A class to handle event labelling functionality.
    Processes category titles and generates appropriate Arabic labels.
    """

    def __init__(self) -> None:
        """Initialize the EventLabResolver with default values."""
        self.foot_ballers: bool = False

    def _handle_special_suffixes(self, category3: str) -> Tuple[str, str, bool]:
        """
        Handle categories with special suffixes like episodes or templates.

        Args:
            category3 (str): The lowercase category string

        Returns:
            Tuple[str, str, bool]: List of category, updated category3, and whether Wikidata was found
        """

        list_of_cat: str = ""

        if category3.endswith(" episodes"):
            list_of_cat, category3 = get_episodes(category3)

        elif category3.endswith(" templates"):
            list_of_cat, category3 = get_templates_fo(category3)

        else:
            # Process with the main category processing function
            list_of_cat, self.foot_ballers, category3 = get_list_of_and_cat3(
                category3, find_stubs=app_settings.find_stubs
            )

        return list_of_cat, category3

    def _get_country_based_label(self, original_category3: str, list_of_cat: str) -> Tuple[str, str]:
        """
        Get country-based labels for specific categories like basketball players.

        Args:
            original_category3 (str): The original category string
            list_of_cat (str): Current list of category value

        Returns:
            Tuple[str, str]: Updated category label and list of category
        """
        category_lab: str = ""

        # ايجاد تسميات مثل لاعبو  كرة سلة أثيوبيون (Find labels like Ethiopian basketball players)
        if list_of_cat == "لاعبو {}":
            category_lab = (
                country_2_title_work(original_category3)
                or wrap_lab_for_country2(original_category3)
                or ye_ts_bot.translate_general_category(original_category3, start_get_country2=False, fix_title=False)
                or get_pop_All_18(original_category3.lower(), "")
                or ""
            )
            if category_lab:
                list_of_cat = ""

        return category_lab, list_of_cat

    def _apply_general_label_functions(self, category3: str) -> str:
        """
        Apply various general label functions in sequence.

        Args:
            category3 (str): The category string to process

        Returns:
            str: The processed category label or empty string
        """
        # Try different label functions in sequence
        category_lab: str = univer.te_universities(category3)
        if category_lab:
            return category_lab

        category_lab = time_to_arabic.convert_time_to_arabic(category3)
        if category_lab:
            return category_lab

        category_lab = wrap_team_xo_normal_2025_with_ends(category3) or new_resolvers_all(category3)
        if category_lab:
            return category_lab

        category_lab = get_pop_All_18(category3, "")
        if category_lab:
            return category_lab

        # If no label found yet, try general translation
        if not category_lab:
            category_lab = ye_ts_bot.translate_general_category(f"category:{category3}", fix_title=False)

        if category_lab:
            return category_lab

        category_lab = (
            country_2_title_work(category3)
            or wrap_lab_for_country2(category3)
            or ye_ts_bot.translate_general_category(category3, start_get_country2=False, fix_title=False)
            or get_pop_All_18(category3.lower(), "")
            or ""
        )
        if category_lab:
            return category_lab

        return category_lab

    def _handle_suffix_patterns(self, category3: str) -> Tuple[str, str]:
        """
        Handle categories that match predefined suffix patterns.

        Args:
            category3 (str): The category string to process

        Returns:
            Tuple[str, str]: List of category and updated category string
        """
        list_of_cat: str = ""

        for pri_ff, vas in combined_suffix_mappings.items():
            suffix = pri_ff.lower()
            if category3.endswith(suffix):
                logger.info(f'>>>><<lightblue>> category3.endswith pri_ff("{pri_ff}")')
                list_of_cat = vas
                category3 = category3[: -len(suffix)].strip()
                break

        return list_of_cat, category3

    def _process_list_category(self, cate_r: str, category_lab: str, list_of_cat: str) -> str:
        """
        Process list categories and format them appropriately.

        Args:
            cate_r (str): Original category string
            category_lab (str): Current category label
            list_of_cat (str): List of category template

        Returns:
            str: Updated category label
        """
        if not list_of_cat or not category_lab:
            return category_lab

        if self.foot_ballers:
            category_lab = list_of_cat_func_foot_ballers(cate_r, category_lab, list_of_cat)
        else:
            category_lab = list_of_cat_func_new(cate_r, category_lab, list_of_cat)

        return category_lab

    def process_category(self, category3: str, cate_r: str) -> str:
        """
        Main method to process a category and return its Arabic label.

        Args:
            cate_r (str): The raw category string to process

        Returns:
            str: The Arabic label for the category
        """
        original_category3 = category3

        # First, try to get squad-related labels
        category_lab = ""

        # Initialize flags
        self.foot_ballers = False
        list_of_cat = ""

        # Handle special suffixes
        if not category_lab:
            list_of_cat, category3 = self._handle_special_suffixes(category3)

        # Handle country-based labels (e.g., basketball players from a country)
        if not category_lab and list_of_cat:
            country_lab, list_of_cat = self._get_country_based_label(original_category3, list_of_cat)
            if country_lab:
                category_lab = country_lab

        # Apply various general label functions
        if not category_lab:
            category_lab = self._apply_general_label_functions(category3)

        # Handle categories that match predefined suffix patterns
        if not category_lab and not list_of_cat:
            list_of_cat, category3 = self._handle_suffix_patterns(category3)

        # Process with event_label_work if no label found yet
        if not category_lab:
            category_lab = event_label_work(category3)

        # Process list categories if both exist
        if list_of_cat and category_lab:
            # Debug before calling list_of_cat_func_new
            if not isinstance(category_lab, str):
                logger.error(f"[BUG] category_lab is dict for cate_r={cate_r} value={category_lab}")
                raise TypeError(f"category_lab must be string, got {type(category_lab)}: {category_lab}")

            category_lab = self._process_list_category(cate_r, category_lab, list_of_cat)

        # Handle case where list exists but no label
        if list_of_cat and not category_lab:
            list_of_cat = ""
            category_lab = event_label_work(original_category3)

        # Try template processing if no label yet
        if not category_lab:
            category_lab = tmp_bot.Work_Templates(original_category3)

        # Try general translation again if still no label
        if not category_lab:
            category_lab = ye_ts_bot.translate_general_category(original_category3, fix_title=False)

        return category_lab


# Create global instance for backward compatibility


resolver = EventLabResolver()


def _finalize_category_label(category_lab: str, cate_r: str) -> str:
    """
    Finalize the category label by applying final formatting.

    Args:
        category_lab (str): The current category label
        cate_r (str): Original category string

    Returns:
        str: The final formatted category label
    """
    if category_lab:
        # Apply final formatting and prefix
        fixed = fixtitle.fixlabel(category_lab, en=cate_r)
        category_lab = f"تصنيف:{fixed}"

    return category_lab


def _handle_cricketer_categories(category3: str) -> str:
    """
    Handle special cricket player categories.

    Args:
        category3 (str): The lowercase category string

    Returns:
        str: The processed category label or empty string
    """
    category32: str = ""
    list_of_cat2: str = ""

    if category3.endswith(" cricketers"):
        list_of_cat2 = "لاعبو كريكت من {}"
        category32 = category3[: -len(" cricketers")]
    elif category3.endswith(" cricket captains"):
        list_of_cat2 = "قادة كريكت من {}"
        category32 = category3[: -len(" cricket captains")]

    if list_of_cat2 and category32:
        category3_lab = get_from_new_p17_final(category32)
        if category3_lab:
            return list_of_cat2.format(category3_lab)

    return ""


def _process_category_formatting(category: str) -> str:
    """
    Process and format the input category string.

    Args:
        category (str): The raw category string

    Returns:
        str: lowercase version without prefix
    """
    if category.startswith("category:"):
        category = category.split("category:")[1]

    category = change_cat(category)

    return category


def event_Lab(cate_r: str) -> str:
    """
    Backward compatibility function that wraps the EventLabResolver class.

    Args:
        cate_r (str): The raw category string to process

    Returns:
        str: The Arabic label for the category
    """
    cate_r = cate_r.lower().replace("_", " ")
    category3: str = _process_category_formatting(cate_r)

    result = ""

    if not result:
        result = resolver.process_category(cate_r, category3)

    # Handle cricket player categories
    if not result:
        result = _handle_cricketer_categories(category3)

    if not result:
        return ""

    result = _finalize_category_label(result, cate_r)
    if result == "تصنيف:":
        return ""

    return result



#!/usr/bin/python3
"""
!
"""

translation_category_relations = {
    "for-deaf": "للصم",
    "for-deafblind": "للصم وللمكفوفون",
    "for-blind": "للمكفوفون",
    "manufactured-by": "صنعتها",
    "manufactured by": "صنعتها",
    "manufactured in": "صنعت في",
    "manufactured-in": "صنعت في",
    "published-by": "نشرتها",
    "published by": "نشرتها",
    "published in": "نشرت في",
    "launched in": "أطلقت في",
    "imprisoned-in": "مسجونون في",
    "imprisoned in": "مسجونون في",
    "launched-in": "أطلقت في",
    "launched by": "أطلقتها",
    "launched-by": "أطلقتها",
    "transferred-from": "نقلت من",
    "transferred from": "نقلت من",
    "built-in": "بنيت في",
    "built in": "بنيت في",
    "built by": "بنتها",
    "built-by": "بنتها",
    "caused by": "بسبب",
    "who defected to": "انشقوا إلى",
    "regarding": "عن",
    "spies for": "لصالح",
    "concerning": "بشأن",
    "shot dead-by-law enforcement officers in": "قتلوا برصاص ضباط إنفاذ القانون في",
    "with physical disabilities": "بإعاقات جسدية",
    "qualification for": "تصفيات مؤهلة إلى",
    "with screenplays by": "كتب نصها السينمائي",
    "with disabilities": "بإعاقات",
    "concluded in": "أبرمت في",
    "used in": "تستخدم في",
    "entered into force in": "دخلت حيز التنفيذ في",
    "invented in": "اخترعت في",
    # "introduced in": "استحدثت في",
    "introduced in": "عرضت في",
    "that uses": "تستخدم",
    "using": "تستخدم",
    "created in": "أنشئت في",
    "written by": "كتبها",
    "opened in": "افتتحت في",
    "originating in": "نشأت في",
    "of works by": "أعمال بواسطة",
    "named after": "سميت بأسماء",
    "that closed in": "أغلقت في",
    "conducted by": "نفذت بواسطة",
    "acquired by": "حصل عليها",
    "destroyed during": "دمرت خلال",
    "during": "خلال",
    "written": "كتبت",
    "closed in": "أغلقت في",
    "demolished in": "هدمت في",
    "in-sport-in": "في الرياضة في",
    "in-sports-in": "في الرياضة في",
    "by language of": "حسب لغة",
    "by-firearm-in": "بإطلاق النار في",
    "by-car-bomb": "بسيارة مفخخة",
    "who died in": "توفوا في",
    "killed while": "قتلوا أثناء",
    "related to": "متعلقة ب",
    "designated for": "خصصت ل",
    "suspended due to": "معلقة بسبب",
    "curtailed and voided due to": "اختصرت وألغيت بسبب",
    "curtailed due to": "اختصرت بسبب",
    "voided due to": "ألغيت بسبب",
    "cancelled due to": "ألغيت بسبب",
    "postponed due to": "تأجلت بسبب",
    "killed in": "قتلوا في",
    "murdered in": "قتلوا في",
    "sentenced to": "حكم عليهم",
    # "sentenced-to-death" : "حكم عليهم بالإعدام",
    "executed-burning by": "أعدموا شنقاً من قبل",
    "executed-hanging by": "أعدموا حرقاً من قبل",
    "executed-decapitation by": "أعدموا بقطع الرأس من قبل",
    "executed-firearm by": "أعدموا بسلاح ناري من قبل",
    "who served in": "خدموا في",
    "collaborators with": "متعاونون مع",
    "charged with": "أتهموا بتهمة",
    "executed for treason against": "أعدموا بتهمة الخيانة العظمى ضد",
    "executed for": "أعدموا بتهمة",
    # "executed by" : "أعدمتهم",
    "executed by": "أعدموا من قبل",
    "deported": "تم ترحيلهم",
    "executed": "أعدموا",
    "disestablished in": "انحلت في",
    # "established in" : "أنشئت في",
    "reestablished in": "أعيد تأسيسها في",
    "involved in": "مرتبطة مع",
    "established in": "أسست في",
    "collapses in": "انهارت في",
    "associated with": "مرتبطة مع",  # مرتبطين مع
    "presented by": "قدمها",
    "directed by": "أخرجها",
    "named by": "سماها",
    "scored by": "سجلها",
    # "hosted by" : "تستضيفها",
    "hosted by": "استضافتها",
    "discontinued in": "توقفت في",
    "developed in": "مطورة في",
    "described in": "وصفت في",
    "discovered in": "اكتشفت في",
    "completed in": "اكتملت في",
    "scheduled for": "مقررة في",
    "headquartered in": "مقرها الرئيسي في",
    "based in": "مقرها في",
    # "remade in" : "أعيد إنتاجها في",
    "remade-in": "أعيد إنتاجها في",
    # "based on" : "مبنية على",
    "basedon": "مبنية على",
    "set in": "تقع أحداثها في",
    "set on": "تقع أحداثها على",
    "filmed in": "صورت في",
    "shot in": "مصورة في",
    "adapted into": "تم تحويلها إلى",  #
    "adapted for": "تم تحويلها إلى",  #
    "recorded in": "سُجلت في",
    "recorded at": "سُجلت في",
    "produced in": "أنتجت في",
    "produced by": "من إنتاج",
    # "impacted by": "تأثرت بـ",
    "who compete in": "يتنافسون في",
    "extended to": "امتدت إلى",
    "involving the": "تشمل",
    "involving": "تشمل",
    "between": "بين",
    "into": "إلى",
    "to": "إلى",
    "by": "حسب",
    "in": "في",
    "elections in": "انتخابات في",
    "from": "من",
    "of": "",
    "for": "في",
    "at": "",
    # "on" : "على",
    "on": "في",
    "about": "عن",
    "outside": "خارج",
    "under": "تحت",
    "against": "ضد",
    # "and": "و",
}

__all__ = [
    "translation_category_relations",
]



#!/usr/bin/python3
"""
TODO: need improvements
"""

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats.new_resolvers.sports_resolvers.nationalities_and_sports import resolve_nats_sport_multi_v2

data_1 = {
    "yemeni domestic basketball": "كرة سلة يمنية محلية",
    "yemeni domestic womens basketball": "كرة سلة يمنية محلية للسيدات",
    "chinese indoor boxing clubs": "أندية بوكسينغ صينية داخل الصالات",
    "chinese indoor boxing coaches": "مدربو بوكسينغ صينية داخل الصالات",
    "chinese indoor boxing competitions": "منافسات بوكسينغ صينية داخل الصالات",
    "chinese indoor boxing leagues": "دوريات بوكسينغ صينية داخل الصالات",
    "chinese outdoor boxing coaches": "مدربو بوكسينغ صينية في الهواء الطلق",
    "chinese outdoor boxing competitions": "منافسات بوكسينغ صينية في الهواء الطلق",
    "chinese outdoor boxing leagues": "دوريات بوكسينغ صينية في الهواء الطلق",
}

nat_p17_oioi_to_check_data = {
    "chinese current boxing seasons": "مواسم بوكسينغ صينية حالية",
    "chinese defunct indoor boxing clubs": "أندية بوكسينغ صينية داخل الصالات سابقة",
    "chinese defunct indoor boxing coaches": "مدربو بوكسينغ صينية داخل الصالات سابقة",
    "chinese defunct indoor boxing competitions": "منافسات بوكسينغ صينية داخل الصالات سابقة",
    "chinese defunct indoor boxing leagues": "دوريات بوكسينغ صينية داخل الصالات سابقة",
    "chinese defunct boxing clubs": "أندية بوكسينغ صينية سابقة",
    "chinese defunct boxing coaches": "مدربو بوكسينغ صينية سابقة",
    "chinese defunct boxing competitions": "منافسات بوكسينغ صينية سابقة",
    "chinese defunct boxing leagues": "دوريات بوكسينغ صينية سابقة",
    "chinese defunct outdoor boxing clubs": "أندية بوكسينغ صينية في الهواء الطلق سابقة",
    "chinese defunct outdoor boxing coaches": "مدربو بوكسينغ صينية في الهواء الطلق سابقة",
    "chinese defunct outdoor boxing competitions": "منافسات بوكسينغ صينية في الهواء الطلق سابقة",
    "chinese defunct outdoor boxing leagues": "دوريات بوكسينغ صينية في الهواء الطلق سابقة",
    "chinese professional boxing clubs": "أندية بوكسينغ صينية للمحترفين",
    "chinese professional boxing coaches": "مدربو بوكسينغ صينية للمحترفين",
    "chinese professional boxing competitions": "منافسات بوكسينغ صينية للمحترفين",
    "chinese professional boxing leagues": "دوريات بوكسينغ صينية للمحترفين",
    "chinese boxing chairmen and investors": "رؤساء ومسيرو البوكسينغ الصينية",
    "chinese boxing leagues": "دوريات البوكسينغ الصينية",
    "chinese boxing clubs": "أندية البوكسينغ الصينية",
    "chinese boxing coaches": "مدربو البوكسينغ الصينية",
    "chinese boxing competitions": "منافسات البوكسينغ الصينية",
    "chinese domestic women's boxing clubs": "أندية بوكسينغ صينية محلية للسيدات",
    "chinese domestic women's boxing coaches": "مدربو بوكسينغ صينية محلية للسيدات",
    "chinese domestic women's boxing competitions": "منافسات بوكسينغ صينية محلية للسيدات",
    "chinese domestic women's boxing leagues": "دوريات بوكسينغ صينية محلية للسيدات",
    "chinese domestic boxing": "بوكسينغ صينية محلية",
    "chinese domestic boxing clubs": "أندية بوكسينغ صينية محلية",
    "chinese domestic boxing coaches": "مدربو بوكسينغ صينية محلية",
    "chinese domestic boxing competitions": "منافسات بوكسينغ صينية محلية",
    "chinese domestic boxing leagues": "دوريات بوكسينغ صينية محلية",
}

data_3 = {}

to_test = [
    ("test_need_improvements_1", data_1, resolve_nats_sport_multi_v2),
    ("test_need_improvements_2", nat_p17_oioi_to_check_data, resolve_nats_sport_multi_v2),
    ("test_need_improvements_3", data_3, resolve_nats_sport_multi_v2),
]


@pytest.mark.parametrize("category, expected", data_1.items(), ids=data_1.keys())
@pytest.mark.fast
def test_need_improvements_1(category: str, expected: str) -> None:
    label2 = resolve_nats_sport_multi_v2(category)
    assert label2 == expected


@pytest.mark.parametrize(
    "category, expected", nat_p17_oioi_to_check_data.items(), ids=nat_p17_oioi_to_check_data.keys()
)
@pytest.mark.fast
def test_need_improvements_2(category: str, expected: str) -> None:
    label2 = resolve_nats_sport_multi_v2(category)
    assert label2 == expected


@pytest.mark.parametrize("category, expected", data_3.items(), ids=data_3.keys())
@pytest.mark.fast
def test_need_improvements_3(category: str, expected: str) -> None:
    label2 = resolve_nats_sport_multi_v2(category)
    assert label2 == expected


@pytest.mark.parametrize("name,data,callback", to_test)
@pytest.mark.dump
def test_dump_it(name: str, data: dict[str, str], callback) -> None:
    expected, diff_result = one_dump_test(data, callback)
    dump_diff(diff_result, name)

    # add_result = {x: v for x, v in data.items() if x in diff_result and "" == diff_result.get(x)}
    # dump_diff(add_result, f"{name}_add")
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""Integration tests for the refactored jobs datasets."""

from __future__ import annotations

from ArWikiCats.translations.jobs.Jobs import Jobs_new, jobs_mens_data
from ArWikiCats.translations.jobs.Jobs2 import JOBS_2, JOBS_3333
from ArWikiCats.translations.jobs.jobs_data_basic import NAT_BEFORE_OCC


def test_jobs_new_contains_female_and_general_entries() -> None:
    """Flattened mapping should expose lowercase keys for combined datasets."""

    assert "film actresses" in Jobs_new
    assert "footballers" in jobs_mens_data


def test_nat_before_occ_includes_religious_expansions() -> None:
    """The nationality-before-occupation list should include religion keys."""

    assert "anglican" in NAT_BEFORE_OCC
    assert "anglicans" in NAT_BEFORE_OCC


def test_jobs2_contains_men_womens_jobs_entries() -> None:
    """``JOBS_2`` should extend the shared MEN_WOMENS_JOBS_2 configuration."""

    sample_keys = [
        "aerospace engineers",
        "archaeologists",
        "biblical scholars",
        "botanists",
        "chemists",
    ]
    for key in sample_keys:
        assert key in JOBS_2
        assert JOBS_2[key]["males"]


def test_jobs2_aliases_are_consistent() -> None:
    """The two exported job datasets should expose distinct objects with data."""

    assert JOBS_2 is not JOBS_3333
    assert JOBS_2
    assert JOBS_3333



"""
Tests
"""

import pytest

from ArWikiCats.new_resolvers.jobs_resolvers.womens import nat_and_gender_keys, womens_resolver_labels
from ArWikiCats.translations import jobs_mens_data, jobs_womens_data

test_data2 = {
    "Category:Canadian women film directors": "تصنيف:مخرجات أفلام كنديات",
    "Category:Canadian women's film directors": "تصنيف:مخرجات أفلام كنديات",
    "Category:Canadian womens film directors": "تصنيف:مخرجات أفلام كنديات",
    "Category:Canadian female film directors": "تصنيف:مخرجات أفلام كنديات",
    # nat
    "female welsh people": "ويلزيات",
    "women's yemeni people": "يمنيات",
    # jobs
    "female eugenicists": "عالمات متخصصات في تحسين النسل",
    "female politicians who committed suicide": "سياسيات أقدمن على الانتحار",
    "female writers people": "كاتبات",
    "female archers": "نبالات",
    # "female football managers": "مديرات كرة قدم",
    "female football managers": "مدربات كرة قدم",
    "actresses": "ممثلات",
    "female actresses": "ممثلات",
    # jobs + expatriate
    "female expatriate football managers": "مدربات كرة قدم مغتربات",
    "expatriate female actresses": "ممثلات مغتربات",
    # nat + jobs
    "yemeni female eugenicists": "عالمات يمنيات متخصصات في تحسين النسل",
    "yemeni female politicians who committed suicide": "سياسيات يمنيات أقدمن على الانتحار",
    "yemeni female contemporary artists": "فنانات يمنيات معاصرات",
    "yemeni actresses": "ممثلات يمنيات",
    "yemeni female writers": "كاتبات يمنيات",
    "greek female writers": "كاتبات يونانيات",
    # "yemeni expatriate female actresses": "ممثلات يمنيات مغتربات",
    "female greek blind": "يونانيات مكفوفات",
    "female writers blind": "كاتبات مكفوفات",
    "female greek writers blind": "كاتبات يونانيات مكفوفات",
    "female writers greek blind": "كاتبات يونانيات مكفوفات",
}


@pytest.mark.parametrize("category,expected", test_data2.items(), ids=test_data2.keys())
def test_nat_pattern_multi(category: str, expected: str) -> None:
    """Test all nat translation patterns."""
    result = womens_resolver_labels(category)
    assert result == expected


@pytest.mark.skip2
def test_must_be_empty() -> None:
    result = womens_resolver_labels("Yemeni singers")
    assert result == ""


@pytest.mark.skip2
def test_must_not_be_empty() -> None:
    result = womens_resolver_labels("Yemeni actresses")
    assert result == "ممثلات يمنيات"


test_religions_data_2 = {
    "Category:Pakistani expatriate female actors": "تصنيف:ممثلات باكستانيات مغتربات",
    "Category:expatriate female actors": "تصنيف:ممثلات مغتربات",
    "women's guitarists": "عازفات قيثارة",
    "women educators": "معلمات",
    "women medical doctors": "طبيبات",
    "women singers": "مغنيات",
    "northern ireland": "",
}


@pytest.mark.parametrize("category,expected", test_religions_data_2.items(), ids=test_religions_data_2.keys())
@pytest.mark.skip2
def test_religions_2(category: str, expected: str) -> None:
    """Test all nat translation patterns."""
    result = womens_resolver_labels(category)
    assert result == expected


def test_nat_and_gender_keys():
    data = nat_and_gender_keys("{en_nat}", "expatriate", "{women}", "{ar_nat} مغتربات")

    assert data == {
        "{en_nat} {women} expatriate": "{ar_nat} مغتربات",
        "{en_nat} expatriate {women}": "{ar_nat} مغتربات",
        "{women} {en_nat} expatriate": "{ar_nat} مغتربات",
    }, print(data)


def test_compare():
    # jobs_mens_data jobs_womens_data
    new_keys = {x: v for x, v in jobs_womens_data.items() if x not in jobs_mens_data}
    expected = {
        "women": "نساء",
        "sportswomen": "رياضيات",
        "midwives": "قابلات",
        "prostitutes": "داعرات",
        "video game actresses": "ممثلات ألعاب فيديو",
        "women's-footballers": "لاعبات كرة قدم",
    }
    # assert new_keys == expected
    assert all(key in jobs_womens_data for key in expected.keys())



"""
Tests
"""

import pytest

from ArWikiCats.legacy_bots.with_years_bot import Try_With_Years

# from ArWikiCats.make_bots.date_bots import with_years_bot


@pytest.mark.fast
def test_try_with_years() -> None:
    # Test basic functionality - should return a string
    result = Try_With_Years("2020 election")
    assert isinstance(result, str)

    # Test with year at end
    result_year_end = Try_With_Years("American Soccer League (1933–83)")
    assert isinstance(result_year_end, str)

    # Test with political term
    result_political = Try_With_Years("116th united states congress")
    assert isinstance(result_political, str)

    # Test empty string
    result_empty = Try_With_Years("")
    assert isinstance(result_empty, str)

    # Test with no year pattern
    result_no_year = Try_With_Years("random category")
    assert isinstance(result_no_year, str)


# ---------------------------------------------------------------------------
# Political terms: _handle_political_terms
# ---------------------------------------------------------------------------


@pytest.mark.parametrize(
    "text,expected",
    [
        ("100th united states congress", "الكونغرس الأمريكي المئة"),
        ("200th united states congress", "الكونغرس الأمريكي المائتين"),
        ("300th united states congress", "الكونغرس الأمريكي الثلاثمائة"),
        ("45th united states congress", "الكونغرس الأمريكي الخامس والأربعون"),
        ("1st iranian majlis", "المجلس الإيراني الأول"),
    ],
)
@pytest.mark.fast
def test_political_terms_mapped_ordinals(text: str, expected: str) -> None:
    result = Try_With_Years(text)
    assert result == expected


@pytest.mark.fast
def test_political_terms_non_matching_returns_empty() -> None:
    # Does not match the political bodies regex, so whole function should return ""
    assert Try_With_Years("5th something else") == ""


# ---------------------------------------------------------------------------
# Year at start: _handle_year_at_start
# ---------------------------------------------------------------------------


@pytest.mark.parametrize(
    "text,expected",
    [
        ("1900 births", "مواليد 1900"),
        ("1999 fires", "حرائق 1999"),
        ("2020 earthquakes", "زلازل 2020"),
    ],
)
@pytest.mark.fast
def test_year_at_start_known_word_after_years(text: str, expected: str) -> None:
    # Uses WORD_AFTER_YEARS directly without get_KAKO/translate
    result = Try_With_Years(text)
    assert result == expected


@pytest.mark.fast
def test_year_at_start_uses_get_kako() -> None:
    # 1900 novels -> not in WORD_AFTER_YEARS, so it should use get_KAKO

    result = Try_With_Years("1900 novels")
    assert result == "روايات 1900"


@pytest.mark.fast
def test_year_at_start_uses_translate_general_category() -> None:
    # Force get_KAKO to return empty so translate_general_category is used

    result = Try_With_Years("1900 protests")
    assert result == "احتجاجات 1900"


@pytest.mark.fast
def test_year_at_start_uses_country2_lab() -> None:
    # get_KAKO and translate_general_category both empty

    result = Try_With_Years("1900 events in Yemen")
    assert result == "أحداث في اليمن 1900"


@pytest.mark.fast
def test_year_at_start_add_in_tabl_separator() -> None:
    # remainder is in Add_in_table -> separator becomes " في "

    result = Try_With_Years("1900 historical documents")
    assert result == "وثائق تاريخية في 1900"


@pytest.mark.fast
def test_year_at_start_ar_lab_before_year_to_add_in() -> None:
    # remainder_label in ar_label_before_year_to_add_in -> " في "

    result = Try_With_Years("1900 rugby union tournaments for national teams")
    assert result == "بطولات اتحاد رجبي في للمنتخبات الوطنية 1900"


@pytest.mark.fast
def test_year_at_start_with_range_and_dash_variants() -> None:
    # Ensure range like "1900–1905" is correctly extracted using RE1/re_sub_year

    # En dash
    result_en_dash = Try_With_Years("1900–1905 earthquakes")
    assert result_en_dash == "زلازل 1900–1905"

    # Minus sign (U+2212) – normalized to '-' in function
    result_minus_sign = Try_With_Years("1900−1905 earthquakes")
    assert result_minus_sign == "زلازل 1900-1905"

    # Normal hyphen for completeness
    result_hyphen = Try_With_Years("1900-1905 earthquakes")
    assert result_hyphen == "زلازل 1900-1905"


# ---------------------------------------------------------------------------
# Year at end (no parentheses): _handle_year_at_end with RE2_compile
# ---------------------------------------------------------------------------


@pytest.mark.fast
def test_year_at_end_simple_year_uses_translate() -> None:
    result = Try_With_Years("earthquakes 1999")
    assert result == "زلازل 1999"


@pytest.mark.fast
def test_year_at_end_range_uses_translate() -> None:
    result = Try_With_Years("earthquakes 1999-2000")
    assert result == "زلازل 1999-2000"


@pytest.mark.fast
def test_year_at_end_uses_country2_when_translate_empty() -> None:
    result = Try_With_Years("floods 2001")
    assert result == "فيضانات 2001"


# ---------------------------------------------------------------------------
# Year at end in parentheses (RE33_compile) and "–present"
# ---------------------------------------------------------------------------


@pytest.mark.fast
def test_year_at_end_parentheses_range() -> None:
    result = Try_With_Years("American Soccer League (1933–83)")
    assert result == "الدوري الأمريكي لكرة القدم (1933–83)"


@pytest.mark.fast
def test_year_at_end_parentheses_with_present() -> None:
    result = Try_With_Years("American Soccer League (1933–present)")
    # "–present" should be converted to "–الآن"
    assert result == "الدوري الأمريكي لكرة القدم (1933–الآن)"


@pytest.mark.fast
def test_year_at_end_parentheses_with_hyphen_variants() -> None:
    # Normal hyphen inside parentheses
    result_hyphen = Try_With_Years("American Soccer League (1933-83)")
    assert result_hyphen == "الدوري الأمريكي لكرة القدم (1933-83)"

    # Minus sign, normalized
    result_minus = Try_With_Years("American Soccer League (1933−83)")
    assert result_minus == "الدوري الأمريكي لكرة القدم (1933-83)"


# ---------------------------------------------------------------------------
# Inputs without any year pattern
# ---------------------------------------------------------------------------


@pytest.mark.parametrize(
    "text",
    [
        "Category:History of Yemen",
        "American films",
        "World War II",  # no trailing 4-digit year or range
        "some random category",
    ],
)
@pytest.mark.fast
def test_no_year_pattern_returns_empty(text) -> None:
    assert Try_With_Years(text) == ""



"""
Tests
"""

from typing import Callable

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats.new_resolvers.countries_names_resolvers.countries_names_v2 import resolve_by_countries_names_v2

political_data = {
    "tunisia political leader": "قادة سياسيون تونسيون",
    "australia political leader": "قادة سياسيون أستراليون",
    "japan political leader": "قادة سياسيون يابانيون",
    "mauritius political leader": "قادة سياسيون موريشيوسيون",
    "morocco political leader": "قادة سياسيون مغاربة",
    "rwanda political leader": "قادة سياسيون روانديون",
    "syria political leader": "قادة سياسيون سوريون",
    "west india political leader": "قادة سياسيون هنود غربيون",
}

main_data = {
    # ar patterns
    "national university of yemen": "جامعة اليمن الوطنية",
    "yemen board members": "أعضاء مجلس اليمن",
    "accidental deaths from falls in tunisia": "وفيات عرضية نتيجة السقوط في تونس",
    "yemen conflict": "نزاع اليمن",
    "yemen cup": "كأس اليمن",
    "oceania cup": "كأس أوقيانوسيا",
    "yemen elections": "انتخابات اليمن",
    "united states elections": "انتخابات الولايات المتحدة",
    "yemen executive cabinet": "مجلس وزراء اليمن التنفيذي",
    "victoria-australia executive cabinet": "مجلس وزراء فيكتوريا (أستراليا) التنفيذي",
    "west india government personnel": "موظفي حكومة الهند الغربية",
    "yemen government": "حكومة اليمن",
    "tunisia government": "حكومة تونس",
    "victoria-australia government": "حكومة فيكتوريا (أستراليا)",
    "georgia government": "حكومة جورجيا",
    "yemen governorate": "محافظة اليمن",
    "yemen presidents": "رؤساء اليمن",
    "tunisia presidents": "رؤساء تونس",
    "west india presidents": "رؤساء الهند الغربية",
    "yemen responses": "استجابات اليمن",
    "west india responses": "استجابات الهند الغربية",
    "tunisia territorial judges": "قضاة أقاليم تونس",
    "democratic-republic-of-congo territorial judges": "قضاة أقاليم جمهورية الكونغو الديمقراطية",
    "tunisia territorial officials": "مسؤولو أقاليم تونس",
    "democratic-republic-of-congo territorial officials": "مسؤولو أقاليم جمهورية الكونغو الديمقراطية",
    "england war and conflict": "حروب ونزاعات إنجلترا",
    "spain war and conflict": "حروب ونزاعات إسبانيا",
    "israel war and conflict": "حروب ونزاعات إسرائيل",
    "yemen war": "حرب اليمن",
    "england war": "حرب إنجلترا",
    "spain war": "حرب إسبانيا",
    "israel war": "حرب إسرائيل",
    "democratic-republic-of-congo war": "حرب جمهورية الكونغو الديمقراطية",
    # squad patterns
    "yemen afc women's asian cup squad": "تشكيلات اليمن في كأس آسيا للسيدات",
    "china afc women's asian cup squad": "تشكيلات الصين في كأس آسيا للسيدات",
    "yemen afc asian cup squad": "تشكيلات اليمن في كأس آسيا",
    "uzbekistan afc asian cup squad": "تشكيلات أوزبكستان في كأس آسيا",
    "victoria-australia fifa world cup squad": "تشكيلات فيكتوريا (أستراليا) في كأس العالم",
    "victoria-australia fifa futsal world cup squad": "تشكيلات فيكتوريا (أستراليا) في كأس العالم لكرة الصالات",
    "democratic-republic-of-congo summer olympics squad": "تشكيلات جمهورية الكونغو الديمقراطية في الألعاب الأولمبية الصيفية",
    "democratic-republic-of-congo winter olympics squad": "تشكيلات جمهورية الكونغو الديمقراطية في الألعاب الأولمبية الشتوية",
    "west india olympics squad": "تشكيلات الهند الغربية في الألعاب الأولمبية",
    # olympics patterns
    "west india summer olympics": "الهند الغربية في الألعاب الأولمبية الصيفية",
    "democratic-republic-of-congo winter olympics": "جمهورية الكونغو الديمقراطية في الألعاب الأولمبية الشتوية",
    # the_female patterns
    "yemen royal air force": "القوات الجوية الملكية اليمنية",
    "tunisia civil war": "الحرب الأهلية التونسية",
    "yemen air force": "القوات الجوية اليمنية",
    "morocco royal defence force": "قوات الدفاع الملكية المغربية",
    "tunisia navy": "البحرية التونسية",
    "morocco royal navy": "البحرية الملكية المغربية",
    "syria naval force": "البحرية السورية",
    "egypt naval forces": "البحرية المصرية",
    # males patterns
    "yemen government officials": "مسؤولون حكوميون يمنيون",
    # the_male patterns
    "yemen premier division": "الدوري اليمني الممتاز",
    "yemen coast guard": "خفر السواحل اليمني",
    "united states congressional delegation": "وفود الكونغرس الأمريكي",
    "united states congressional delegations": "وفود الكونغرس الأمريكي",
    "yemen parliament": "البرلمان اليمني",
    "united states congress": "الكونغرس الأمريكي",
    "england house of commons": "مجلس العموم الإنجليزي",
    "england house-of-commons": "مجلس العموم الإنجليزي",
    "united states senate election": "انتخابات مجلس الشيوخ الأمريكي",
    "united states senate elections": "انتخابات مجلس الشيوخ الأمريكي",
    "iraq fa cup": "كأس الاتحاد العراقي",
    "Bangladesh Federation Cup": "كأس الاتحاد البنغلاديشي",
    "united states marine corps personnel": "أفراد سلاح مشاة البحرية الأمريكي",
    "yemen army personnel": "أفراد الجيش اليمني",
    "united states coast guard aviation": "طيران خفر السواحل الأمريكي",
    "united states abortion law": "قانون الإجهاض الأمريكي",
    "france labour law": "قانون العمل الفرنسي",
    "yemen professional league": "دوري المحترفين اليمني",
    "yemen first division league": "الدوري اليمني الدرجة الأولى",
    "yemen second division": "الدوري اليمني الدرجة الثانية",
    "yemen second division league": "الدوري اليمني الدرجة الثانية",
    "yemen third division league": "الدوري اليمني الدرجة الثالثة",
    "yemen forth division league": "الدوري اليمني الدرجة الرابعة",
}


@pytest.mark.parametrize("category, expected", main_data.items(), ids=main_data.keys())
@pytest.mark.fast
def test_resolve_by_countries_names_v2(category: str, expected: str) -> None:
    label = resolve_by_countries_names_v2(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", political_data.items(), ids=political_data.keys())
@pytest.mark.fast
def test_political_data_v2(category: str, expected: str) -> None:
    label = resolve_by_countries_names_v2(category)
    assert label == expected


TEMPORAL_CASES = [
    ("test_resolve_by_countries_names_v2", main_data, resolve_by_countries_names_v2),
    ("test_political_data_v2", political_data, resolve_by_countries_names_v2),
]


@pytest.mark.dump
@pytest.mark.parametrize("name,data, callback", TEMPORAL_CASES)
def test_all_dump(name: str, data: dict[str, str], callback: Callable) -> None:
    expected, diff_result = one_dump_test(data, callback)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""
Supplementary mappings for educational, sporting and political contexts.
"""

from ...helps import len_print
from ..sports.cycling import CYCLING_TEMPLATES
from ..utils.json_dir import open_json_file
from .keys2 import new_2019

CAMBRIDGE_COLLEGES: dict[str, str] = {
    "christ's": "كريست",
    "churchill": "تشرشل",
    "clare hall": "كلير هول",
    "corpus christi": "كوربوس كريستي",
    "darwin": "داروين",
    "downing": "داونينج",
    "fitzwilliam": "فيتزويليام",
    "girton": "غيرتون",
    "gonville and caius": "غونفيل وكايوس",
    "homerton": "هومرتون",
    "hughes hall": "هيوز هول",
    "jesus": "يسوع",
    "king's": "كينجز",
    "lucy cavendish": "لوسي كافنديش",
    "magdalene": "المجدلية",
    "murray edwards": "موراي إدواردز",
    "newnham": "نونهم",
    "oriel": "أوريل",
    "pembroke": "بمبروك",
    "peterhouse": "بترهووس",
    "queens'": "كوينز",
    "robinson": "روبنسون",
    "selwyn": "سلوين",
    "sidney sussex": "سيدني ساسكس",
    "st catharine's": "سانت كاثارين",
    "st edmund's": "سانت ادموند",
    "st john's": "سانت جونز",
    "trinity hall": "قاعة الثالوث",
    "trinity": "ترينيتي",
    "wolfson": "وولفسون",
}

INTER_FEDERATIONS: dict[str, str] = open_json_file("inter_federations.json")

BATTLESHIP_CATEGORIES: dict[str, str] = {
    "aircraft carriers": "حاملات طائرات",
    "aircrafts": "طائرات",
    "amphibious warfare vessels": "سفن حربية برمائية",
    "auxiliary ships": "سفن مساعدة",
    "battlecruisers": "طرادات معركة",
    "battleships": "بوارج",
    "cargo aircraft": "طائرة شحن",
    "cargo aircrafts": "طائرة شحن",
    "cargo ships": "سفن بضائع",
    "coastal defence ships": "سفن دفاع ساحلية",
    "corvettes": "فرقيطات",
    "cruisers": "طرادات",
    "destroyers": "مدمرات",
    "escort ships": "سفن مرافقة",
    "frigates": "فرقاطات",
    "gunboats": "زوارق حربية",
    "helicopters": "مروحيات",
    "light cruisers": "طرادات خفيفة",
    "mine warfare vessels": "سفن حرب ألغام",
    "minesweepers": "كاسحات ألغام",
    "missile boats": "قوارب صواريخ",
    "naval ships": "سفن قوات بحرية",
    "ocean liners": "عابرات محيطات",
    "passenger ships": "سفن ركاب",
    "patrol vessels": "سفن دورية",
    "radar ships": "سفن رادار",
    "service vessels": "سفن خدمة",
    "Ship classes": "فئات سفن",
    "ships of the line": "سفن الخط",
    "ships": "سفن",
    "sloops": "سلوبات",
    "tall ships": "سفن طويلة",
    "torpedo boats": "زوارق طوربيد",
    "troop ships": "سفن جنود",
    "unmanned aerial vehicles": "طائرات بدون طيار",
    "unmanned military aircraft": "طائرات عسكرية بدون طيار",
}

RELIGIOUS_TRADITIONS: dict[str, dict[str, str]] = {
    "catholic": {"with_al": "الكاثوليكية", "no_al": "كاثوليكية"},
    "eastern orthodox": {"with_al": "الأرثوذكسية الشرقية", "no_al": "أرثوذكسية شرقية"},
    "moravian": {"with_al": "المورافية", "no_al": "مورافية"},
    "orthodox": {"with_al": "الأرثوذكسية", "no_al": "أرثوذكسية"},
}

UNITED_STATES_POLITICAL: dict[str, str] = {
    "united states house of representatives": "مجلس النواب الأمريكي",
    "united states house-of-representatives": "مجلس النواب الأمريكي",
    "united states presidential": "الرئاسة الأمريكية",
    "united states senate": "مجلس الشيوخ الأمريكي",
    "united states vice presidential": "نائب رئيس الولايات المتحدة",
    "united states vice-presidential": "نائب رئيس الولايات المتحدة",
    "vice presidential": "نائب الرئيس",
    "vice-presidential": "نائب الرئيس",
}

INTER_FEDS_LOWER: dict[str, str] = {key.lower(): value for key, value in INTER_FEDERATIONS.items()}


def build_new2019() -> dict[str, str]:
    """Assemble the 2019 key mapping including sports and political data."""

    data = dict(new_2019)

    for college_key, college_label in CAMBRIDGE_COLLEGES.items():
        data[f"{college_key}, Cambridge"] = f"{college_label} (جامعة كامبريدج)"
        data[f"{college_key} College, Cambridge"] = f"كلية {college_label} (جامعة كامبريدج)"
        data[f"{college_key} College, Oxford"] = f"كلية {college_label} جامعة أكسفورد"

    data.update(INTER_FEDS_LOWER)

    data.update({key.lower(): label for key, label in BATTLESHIP_CATEGORIES.items()})
    data.update({f"active {key.lower()}": f"{label} نشطة" for key, label in BATTLESHIP_CATEGORIES.items()})

    for tradition, labels in RELIGIOUS_TRADITIONS.items():
        no_al = labels["no_al"]
        base_key = tradition.lower()
        data[f"{base_key} cathedrals"] = f"كاتدرائيات {no_al}"
        data[f"{base_key} monasteries"] = f"أديرة {no_al}"
        data[f"{base_key} orders and societies"] = f"طوائف وتجمعات {no_al}"
        data[f"{base_key} eparchies"] = f"أبرشيات {no_al}"
        data[f"{base_key} religious orders"] = f"طوائف دينية {no_al}"
        data[f"{base_key} religious communities"] = f"طوائف دينية {no_al}"
        if tradition != "catholic":
            data[f"{base_key} catholic"] = f"{labels['with_al']} الكاثوليكية"
            data[f"{base_key} catholic eparchies"] = f"أبرشيات {no_al} كاثوليكية"

    data.update(CYCLING_TEMPLATES)

    for key, label in UNITED_STATES_POLITICAL.items():
        base_key = key.lower()
        data[f"{base_key} electors"] = f"ناخبو {label}"
        data[f"{base_key} election"] = f"انتخابات {label}"
        data[f"{base_key} elections"] = f"انتخابات {label}"
        data[f"{base_key} candidates"] = f"مرشحو {label}"

    return data


new2019: dict[str, str] = build_new2019()

__all__ = ["new2019", "INTER_FEDS_LOWER"]

len_print.data_len(
    "all_keys4.py",
    {
        "INTER_FEDS_LOWER": INTER_FEDS_LOWER,
        "new2019": new2019,
    },
)



"""

"""

import pytest
from load_one_data import dump_diff, dump_same_and_not_same, one_dump_test

from ArWikiCats.legacy_bots.o_bots.bys import by_people_bot
from ArWikiCats.new_resolvers.bys_new import resolve_by_labels

data_0 = {"by football team": "حسب فريق كرة القدم"}

by_data_peoples = {
    "by abraham lincoln": "بواسطة أبراهام لينكون",
    "by andrea mantegna": "بواسطة أندريا مانتينيا",
    "by benjamin britten": "بواسطة بنجامين بريتن",
    "by bob dylan": "بواسطة بوب ديلن",
    "by béla bartók": "بواسطة بيلا بارتوك",
    "by camille saint-saëns": "بواسطة كامي سان صانز",
    "by carl nielsen": "بواسطة كارل نيلسن",
    "by charles edward stuart": "بواسطة تشارلز إدوارد ستيوارت",
    "by charles mingus": "بواسطة شارليس مينغوس",
    "by don costa": "بواسطة دون كوستا",
    "by donald trump": "بواسطة دونالد ترمب",
    "by edgar degas": "بواسطة إدغار ديغا",
    "by edward elgar": "بواسطة إدوارد إلجار",
    "by edward iv": "بواسطة إدوارد الرابع ملك إنجلترا",
    "by edward viii": "بواسطة إدوارد الثامن ملك المملكة المتحدة",
    "by felix mendelssohn": "بواسطة فيلكس مندلسون",
    "by frank zappa": "بواسطة فرانك زابا",
    "by franklin pierce": "بواسطة فرانكلين بيرس",
    "by frederick douglass": "بواسطة فريدريك دوغلاس",
    "by george gershwin": "بواسطة جورج غيرشوين",
    "by george ii of great britain": "بواسطة جورج الثاني ملك بريطانيا العظمى",
    "by george vi": "بواسطة جورج السادس ملك المملكة المتحدة",
    "by gertrude stein": "بواسطة جيرترود شتاين",
    "by harvey kurtzman": "بواسطة هارفي كورتزمان",
    "by hieronymus bosch": "بواسطة هيرونيموس بوس",
    "by jack london": "بواسطة جاك لندن",
    "by jacob van ruisdael": "بواسطة جاكوب فان روسيدل",
    "by jacques offenbach": "بواسطة جاك أوفنباخ",
    "by jawaharlal nehru": "بواسطة جواهر لال نهرو",
    "by jerome robbins": "بواسطة جيرومي روبين",
    "by jimmy carter": "بواسطة جيمي كارتر",
    "by joe biden": "بواسطة جو بايدن",
    "by johannes brahms": "بواسطة يوهانس برامس",
    "by johannes vermeer": "بواسطة يوهانس فيرمير",
    "by john tyler": "بواسطة جون تايلر",
    "by louis xv": "بواسطة لويس الخامس عشر ملك فرنسا",
    "by louis xvi": "بواسطة لويس السادس عشر ملك فرنسا في فرنسا",
    "by m. r. james": "بواسطة إم. جيمس",
    "by matt damon": "بواسطة مات ديمون",
    "by muhammad": "بواسطة محمد",
    "by nadine gordimer": "بواسطة نادين غورديمير",
    "by napoleon": "بواسطة نابليون",
    "by nikolai rimsky-korsakov": "بواسطة نيكولاي ريمسكي كورساكوف",
    "by norman rockwell": "بواسطة نورمان روكويل",
    "by pablo picasso": "بواسطة بابلو بيكاسو",
    "by pope clement xiv": "بواسطة كليمنت الرابع عشر",
    "by pope gregory xvi": "بواسطة غريغوري السادس عشر",
    "by pope honorius iii": "بواسطة هونريوس الثالث",
    "by pope leo xiii": "بواسطة ليون الثالث عشر",
    "by pope paul vi": "بواسطة بولس السادس",
    "by pope pius xi": "بواسطة بيوس الحادي عشر",
    "by pyotr ilyich tchaikovsky": "بواسطة بيتر إليتش تشايكوفسكي",
    "by queen victoria": "بواسطة الملكة فيكتوريا",
    "by richard strauss": "بواسطة ريتشارد شتراوس",
    "by satyajit ray": "بواسطة ساتياجيت راي",
    "by sergei prokofiev": "بواسطة سيرغي بروكوفييف",
    "by sergei rachmaninoff": "بواسطة سيرجي رخمانينوف",
    "by theodore roosevelt": "بواسطة ثيودور روزفلت",
    "by titian": "بواسطة تيتيان",
    "by truman capote": "بواسطة ترومان كابوتي",
    "by warren g. harding": "بواسطة وارن جي. هاردينغ",
    "by will ferrell": "بواسطة ويل فيرل",
    "by william blake": "بواسطة وليم بليك",
    "by wolfgang amadeus mozart": "بواسطة فولفغانغ أماديوس موتسارت",
}

by_data_fast = {
    "by century": "حسب القرن",
    "by city": "حسب المدينة",
    "by conflict": "حسب النزاع",
    "by county": "حسب المقاطعة",
    "by decade": "حسب العقد",
    "by educational affiliation": "حسب الانتماء التعليمي",
    "by educational institution": "حسب الهيئة التعليمية",
    "by hanging": "بالشنق",
    "by high school": "حسب المدرسة الثانوية",
    "by interest": "حسب الاهتمام",
    "by law enforcement officers": "بواسطة ضباط إنفاذ القانون",
    "by league": "حسب الدوري",
    "by location": "حسب الموقع",
    "by month": "حسب الشهر",
    "by nationality": "حسب الجنسية",
    "by newspaper": "حسب الصحيفة",
    "by occupation": "حسب المهنة",
    "by organization": "حسب المنظمة",
    "by person": "حسب الشخص",
    "by populated place": "حسب المكان المأهول",
    "by province or territory": "حسب المقاطعة أو الإقليم",
    "by province": "حسب المقاطعة",
    "by school": "حسب المدرسة",
    "by stabbing": "بالطعن",
    "by subject": "حسب الموضوع",
    "by team": "حسب الفريق",
    "by university or college": "حسب الجامعة أو الكلية",
    "by violence": "بسبب العنف",
    "by year": "حسب السنة",
}


@pytest.mark.parametrize("category, expected", by_data_fast.items(), ids=by_data_fast.keys())
@pytest.mark.fast
def test_by_data(category: str, expected: str) -> None:
    label = resolve_by_labels(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", by_data_peoples.items(), ids=by_data_peoples.keys())
@pytest.mark.fast
def test_by_data_peoples(category: str, expected: str) -> None:
    label = by_people_bot(category)
    assert label == expected


TEMPORAL_CASES = [
    ("test_by_data_1", by_data_fast, resolve_by_labels),
    ("test_by_data_peoples", by_data_peoples, by_people_bot),
]


@pytest.mark.dump
@pytest.mark.parametrize("name,data, callback", TEMPORAL_CASES)
def test_all_dump(name: str, data: dict[str, str], callback) -> None:
    expected, diff_result = one_dump_test(data, callback)

    dump_diff(diff_result, name)
    dump_same_and_not_same(data, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"
